/* This code tests the functionality of the GeedotPopupFrame widget
 * 
 * Author: Luke Crowthers
 * File: GeedotFrame.c
 * Date: Jan 2001
 * License: GPL
 */

#include <gdtoolkit.h>

Bool grabbed = False;
int grabx = 0;
int graby = 0;

int move_window (GeedotWidget *, XEvent *);
int grab_window (GeedotWidget *, XEvent *);
int release_window (GeedotWidget *);

int main (int args, char *argvals[]) {
	GeedotWidget *gframe;
	GeedotWidget *grabbutton;
	GeedotArea gframe_area;
	GeedotAction gframe_action;
	Display *display;
	char *display_name = (char *) getenv("DISPLAY");
	XEvent event;

	display = XOpenDisplay(display_name);
	gframe_area.x = 200;
	gframe_area.y = 200;
	gframe_area.width = 200;
	gframe_area.height = 200;

	gframe = gdot_add_popup_frame(gdot_get_root_widget(display), gframe_area);
	gdot_show_frame(gframe);
	gframe_area.x = 0;
	gframe_area.y = 0;
	gframe_area.height = 10;
	gframe_area.width = 199;
	grabbutton = gdot_add_button(gframe, gframe_area);
	gdot_redraw_widget(grabbutton);
	gframe_action.trigger_mask = GeedotPointerMotion;
	gframe_action.response = GEEDOT_FUNCTION(move_window);
	gframe_action.response_data = gframe;
	gdot_register_widget_action(grabbutton, gframe_action);

	gframe_action.trigger_mask = GeedotButtonPress;
	gframe_action.response = GEEDOT_FUNCTION(grab_window);
	gdot_register_widget_action(grabbutton, gframe_action);

	gframe_action.trigger_mask = GeedotButtonRelease;
	gframe_action.response = GEEDOT_FUNCTION(release_window);
	gdot_register_widget_action(grabbutton, gframe_action);

	while (1) {
		gdot_wait_for_event(gframe);
		event = gdot_process_next_event(gframe);
		if (event.type == ClientMessage) {
			if (!strcmp("WM_DELETE_WINDOW", XGetAtomName(gframe->display, event.xclient.data.l[0]))) {
				gdot_widget_destroy(gframe);
				exit(0);
			}
		}
	}
	return 0;
}

int move_window (GeedotWidget *frame, XEvent *event) {
	int x, y, buf;
	Window root;

	XQueryPointer(frame->display, frame->drawable, &root, &root, &x, &y, &buf, &buf, &buf);

	if (grabbed) {
		frame->area.x = x - grabx;
		frame->area.y = y - graby;
		XMoveWindow(frame->display, frame->drawable, frame->area.x, frame->area.y);
	}
	return 0;
}

int grab_window (GeedotWidget *frame, XEvent *event) {
	grabx = event->xbutton.x;
	graby = event->xbutton.y;
	grabbed = True;
}

int release_window (GeedotWidget *frame) {
	grabbed = False;
}
