/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitbutton.c - button creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Aug, 2000
 *        Modified for new GeedotWidget handling methods: Dec, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>

GeedotWidget *gdot_add_button (GeedotWidget *frame, GeedotArea area) {
	GeedotAction  action_temp;
	GeedotWidget *last_widget;
	int font_char_width = XTextWidth(frame->font, " ", 1);
	int font_char_height = frame->font->ascent + frame->font->descent;

	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->type = GEEDOT_BUTTON;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_button);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	if (area.width < 0) {
		frame->last_widget->widget->area.width = XTextWidth(frame->font, "", strlen("")) + 5;
	}
	else {
		frame->last_widget->widget->area.width = area.width;
	}
	if (area.height < 0) {
		frame->last_widget->widget->area.height = font_char_height + 4;
	}
	else {
		frame->last_widget->widget->area.height = area.height;
	}

	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;

	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;

	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;

	frame->last_widget->widget->widget_data = (GeedotButton *) malloc(sizeof(GeedotButton));
	GEEDOT_BUTTON(frame->last_widget->widget->widget_data)->type = GEEDOT_TEXT_BUTTON;
	GEEDOT_BUTTON(frame->last_widget->widget->widget_data)->label = (char *) malloc(strlen("") * sizeof(char) + 1);
	strcpy(GEEDOT_BUTTON(frame->last_widget->widget->widget_data)->label, "");

	action_temp.trigger_mask = GeedotButtonPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_button_set_depressed);
	action_temp.response_data = frame->last_widget->widget;
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	action_temp.trigger_mask = GeedotButtonPressRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_button_set_released);
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	last_widget = frame->last_widget->widget;

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;

	return last_widget;
}

GeedotWidget *gdot_add_text_button (GeedotWidget *frame, GeedotArea area, char *text) {
	GeedotWidget *widget;
	
	widget = gdot_add_button(frame, area);
	gdot_button_set_label(widget, text);
	gdot_redraw_widget(widget);
	return widget;
}

GeedotWidget *gdot_add_pixmap_button (GeedotWidget *frame, GeedotArea area, Pixmap pixmap) {
	GeedotWidget *widget;
	unsigned int x, y, width, height, buffer;
	Window root_holder;
	
	XGetGeometry(frame->display, pixmap, &root_holder, &x, &y, &width, &height, &buffer, &buffer);
	if (area.width < 0) {
		area.width = width + 4;
	}
	if (area.height < 0) {
		area.height = height + 4;
	}
	widget = gdot_add_button(frame, area);
	GEEDOT_BUTTON(widget->widget_data)->type = GEEDOT_PIXMAP_BUTTON;
	free(GEEDOT_BUTTON(widget->widget_data)->label);
	gdot_button_set_pixmap(widget, pixmap);
	gdot_redraw_widget(widget);
	return widget;
}

GeedotWidget *gdot_add_text_pixmap_button (GeedotWidget *frame, GeedotArea area, char *text, Pixmap pixmap) {
	GeedotWidget *widget;
	unsigned int x, y, width, height, buffer;
	Window root_holder;
	
	XGetGeometry(frame->display, pixmap, &root_holder, &x, &y, &width, &height, &buffer, &buffer);
	if (area.width < 0) {
		area.width = XTextWidth(frame->font, text, strlen(text)) + width + 4;
	}
	if (area.height < 0) {
		area.height = height + 4;
		if (area.height < frame->font->ascent + frame->font->descent + 4) {
			area.height = frame->font->ascent + frame->font->descent + 4;
		}
	}
	
	widget = gdot_add_button(frame, area);
	GEEDOT_BUTTON(widget->widget_data)->type = GEEDOT_TEXT_PIXMAP_BUTTON;
	gdot_button_set_pixmap(widget, pixmap);
	gdot_button_set_label(widget, text);
	gdot_redraw_widget(widget);
	return widget;
}

int gdot_redraw_button (GeedotWidget *widget) {
	if (gdot_widget_has_motion_focus(widget) && widget->is_active) {
		if (gdot_widget_has_focus(widget)) {
			if (*widget->pointer_state & BUTTON1_DEPRESSED) {
				gdot_draw_button_depressed(widget);
			}
			else {
				gdot_draw_button_highlighted(widget);
			}
		}
		else {
			if (!(*widget->pointer_state & BUTTON1_DEPRESSED)) {
				gdot_draw_button_highlighted(widget);
			}
		}
	}
	else {
		gdot_draw_button_plain(widget);
	}
	
	if (!widget->is_active) {
		gdot_shade_widget(widget);
	}
}

int gdot_draw_button_highlighted (GeedotWidget *widget) {
	Window root_holder;
	unsigned int x, y, width, height, border, depth;
	unsigned int absx, absy;
	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, widget->area.width - 1, widget->area.height - 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy, absx + widget->area.width - 1, absy);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy + 1, absx, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + widget->area.height - 1, absx + widget->area.width - 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width - 1, absy + widget->area.height - 2, absx + widget->area.width - 1, absy + 2);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy + 1, absx + widget->area.width, absy + widget->area.height - 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy + 1, absx + widget->area.width, absy + widget->area.height - 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_BUTTON) {
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4 /*- widget->font->max_bounds.lbearing*/, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &border, &depth);
		XCopyArea(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, widget->backbuffer, widget->frame_gc, x, y, width, height, absx + (widget->area.width - width) / 2 + 1, absy + (widget->area.height - height) / 2 + 1);
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &border, &depth);
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4 /*- widget->font->max_bounds.lbearing*/, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
		XCopyArea(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, widget->backbuffer, widget->frame_gc, x, y, width, height, absx + (widget->area.width + XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) - width) / 2 + 1, absy + (widget->area.height - height) / 2 + 1);
	}
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_draw_button_depressed (GeedotWidget *widget) {
	Window root_holder;
	unsigned int x, y, width, height, border, depth;
	unsigned int absx, absy;
	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, widget->area.width - 1, widget->area.height - 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy, absx + widget->area.width - 1, absy);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy + 1, absx, absy + widget->area.height - 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, absx + 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 1, absx + widget->area.width - 1, absy + 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + widget->area.height - 1, absx + widget->area.width - 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width - 1, absy + widget->area.height - 2, absx + widget->area.width - 1, absy + 2);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy + 1, absx + widget->area.width, absy + widget->area.height - 1);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_BUTTON) {
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4 /*- widget->font->max_bounds.lbearing*/, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &border, &depth);
		XCopyArea(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, widget->backbuffer, widget->frame_gc, x, y, width, height, absx + (widget->area.width - width) / 2 + 1, absy + (widget->area.height - height) / 2 + 1);
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &border, &depth);
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4 /*- widget->font->max_bounds.lbearing*/, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
		XCopyArea(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, widget->backbuffer, widget->frame_gc, x, y, width, height, absx + (widget->area.width + XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) - width) / 2 + 1, absy + (widget->area.height - height) / 2 + 1);
	}
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_draw_button_plain (GeedotWidget *widget) {
	Window root_holder;
	unsigned int x, y, width, height, border, depth;
	unsigned int absx, absy;
	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 2, widget->area.width - 2, widget->area.height - 2);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy, absx + widget->area.width - 1, absy);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy + 1, absx, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + widget->area.height - 1, absx + widget->area.width - 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width - 1, absy + widget->area.height - 2, absx + widget->area.width - 1, absy + 2);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy + 1, absx + widget->area.width, absy + widget->area.height - 1);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, absx + 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 1, absx + widget->area.width - 1, absy + 1);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_BUTTON) {
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4 /*- widget->font->max_bounds.lbearing*/, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &border, &depth);
		XCopyArea(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, widget->backbuffer, widget->frame_gc, x, y, width, height, absx + (widget->area.width - width) / 2 + 1, absy + (widget->area.height - height) / 2 + 1);
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &border, &depth);
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4 /*- widget->font->max_bounds.lbearing*/, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
		XCopyArea(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, widget->backbuffer, widget->frame_gc, x, y, width, height, absx + (widget->area.width + XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) - width) / 2 + 1, absy + (widget->area.height - height) / 2 + 1);
	}
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_button_set_depressed (GeedotWidget *widget) {
	if (widget->is_active) {
		*widget->pointer_state = BUTTON1_DEPRESSED;
		gdot_redraw_widget(widget);
	}
}

int gdot_button_set_released (GeedotWidget *widget) {
	if (widget->is_active) {
		*widget->pointer_state = 0;
		gdot_redraw_widget(widget);
	}
}

int gdot_button_set_label (GeedotWidget *widget, char *label) {
	GEEDOT_BUTTON(widget->widget_data)->label = (char *) realloc(GEEDOT_BUTTON(widget->widget_data)->label, (strlen(label) + 1) * sizeof(char));
	strcpy(GEEDOT_BUTTON(widget->widget_data)->label, label);
	gdot_button_fit(widget);
	gdot_redraw_widget(widget);
}

int gdot_button_set_pixmap (GeedotWidget *widget, Pixmap pixmap) {
	GEEDOT_BUTTON(widget->widget_data)->pixmap = pixmap;
	gdot_button_fit(widget);
	gdot_redraw_widget(widget);
}

int gdot_button_fit (GeedotWidget *widget) {
	unsigned int x, y, width, height, buffer;
	Window root_holder;
	
	if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_TEXT_BUTTON) {
		if (widget->area.width < XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) + 4) {
			widget->area.width = XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) + 6;
		}
		if (widget->area.height < widget->font->ascent + widget->font->descent + 6) {
			widget->area.height = widget->font->ascent + widget->font->descent + 6;
		}
		return 0;
	}
	else if (GEEDOT_BUTTON(widget->widget_data)->type == GEEDOT_PIXMAP_BUTTON) {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &buffer, &buffer);
		if (widget->area.width < width + 4) {
			widget->area.width = width + 4;
		}
		if (widget->area.height < height + 4) {
			widget->area.height = height + 4;
		}
		return 0;
	}
	else {
		XGetGeometry(widget->display, GEEDOT_BUTTON(widget->widget_data)->pixmap, &root_holder, &x, &y, &width, &height, &buffer, &buffer);
		if (widget->area.width < width + XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) + 4) {
			widget->area.width = width + XTextWidth(widget->font, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label)) + 4;
		}
		if (height < widget->font->ascent + widget->font->descent) {
			if (widget->area.height < widget->font->ascent + widget->font->descent + 4) {
				widget->area.height = widget->font->ascent + widget->font->descent + 4;
			}
		}
		else {
			if (widget->area.height < height + 4) {
				widget->area.height = height + 4;
			}
		}
		return 0;
	}
}
