/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitcheckbox.c - checkbox creation, drawing and handling
 *   Author: Luke Crowthers
 *   Start Date: Sep, 2000
 *        Modified for new GeedotWidget handling methods: Jan, 2001
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>

int gdot_checkbox_grab_focus (GeedotWidget *);

GeedotWidget *gdot_add_checkbox (GeedotWidget *frame, GeedotArea area) {
	GeedotAction action_temp;
	GeedotWidget *last_widget;
	int font_char_width = XTextWidth(frame->font, " ", 1);
	int font_char_height = frame->font->ascent + frame->font->descent;
	
	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->type = GEEDOT_CHECKBOX;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_checkbox);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	if (area.width < 0) {
		frame->last_widget->widget->area.width = font_char_width * strlen("") + 13;
	}
	else {
		frame->last_widget->widget->area.width = area.width;
	}
	if (area.height < 0) {
		frame->last_widget->widget->area.height = font_char_height;
	}
	else {
		frame->last_widget->widget->area.height = area.height;
	}
	
	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;
	
	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;
	
	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;
	
	frame->last_widget->widget->widget_data = (GeedotCheckbox *) malloc(sizeof(GeedotCheckbox));
	GEEDOT_CHECKBOX(frame->last_widget->widget->widget_data)->is_checked = False;
	GEEDOT_CHECKBOX(frame->last_widget->widget->widget_data)->label = (char *) malloc(1 * sizeof(char));
	strcpy(GEEDOT_CHECKBOX(frame->last_widget->widget->widget_data)->label, "");
	
	action_temp.trigger_mask = GeedotButtonPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_checkbox_set_depressed);
	action_temp.response_data = frame->last_widget->widget;
	gdot_register_widget_action(frame->last_widget->widget, action_temp);
	
	action_temp.trigger_mask = GeedotButtonRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_checkbox_set_selected);
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	last_widget = frame->last_widget->widget;

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;
	
	gdot_redraw_widget(last_widget);
	
	return last_widget;
}

int gdot_redraw_checkbox (GeedotWidget *widget) {
	if (gdot_widget_has_motion_focus(widget) && widget->is_active) {
		if (gdot_widget_has_focus(widget)) {
			if (*widget->pointer_state & BUTTON1_DEPRESSED) {
				gdot_draw_checkbox_depressed(widget);
			}
			else {
				gdot_draw_checkbox_highlighted(widget);
			}
		}
		else {
			if (!(*widget->pointer_state & BUTTON1_DEPRESSED)) {
				gdot_draw_checkbox_highlighted(widget);
			}
		}
	}
	else {
		gdot_draw_checkbox_plain(widget);
	}
	
	if (!widget->is_active) {
		gdot_shade_widget(widget);
	}
}

int gdot_draw_checkbox_highlighted (GeedotWidget *widget) {
	int box_y;
	unsigned int absx, absy;

	box_y = gdot_widget_absolute_y(widget);
	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	}

	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 1, 7, 9);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 2, absx + 1, box_y + 8);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 2, absx + 9, box_y + 8);

	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	}

	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, box_y + 2, absx, box_y + 8);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y, absx + 8, box_y);

	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 1);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 1);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 9);

	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	}

	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 10, absx + 8, box_y + 10);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 10, box_y + 2, absx + 10, box_y + 8);

	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 9);

	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid2.color.pixel);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 2);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 5);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 5, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 5, box_y + 5);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 2);

		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark2.color.pixel);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 2);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 5, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 5);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 7);

		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 2);
	}
	else {      
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 2, absx + 1, absy + 8);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 1, absx + 8, absy + 1);

		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 9, absx + 8, absy + 9);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, absy + 2, absx + 9, absy + 8);
	}
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 12, absy, widget->area.width - 12, widget->area.height);
	XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
	XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 13, box_y + 10, GEEDOT_CHECKBOX(widget->widget_data)->label, strlen(GEEDOT_CHECKBOX(widget->widget_data)->label));
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_draw_checkbox_depressed (GeedotWidget *widget) {
	int box_y;
	unsigned int absx, absy;
	
	box_y = gdot_widget_absolute_y(widget);
	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 1, 7, 9);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 2, absx + 1, box_y + 8);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 2, absx + 9, box_y + 8);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, box_y + 2, absx, box_y + 8);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y, absx + 8, box_y);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 1);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 1);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 9);
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);   
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 10, absx + 8, box_y + 10);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 10, box_y + 2, absx + 10, box_y + 8);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 9);
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_draw_checkbox_plain (GeedotWidget *widget) {
	int box_y;
	unsigned int absx, absy;
	
	box_y = gdot_widget_absolute_y(widget);
	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);
	
	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	}
	
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 1, 7, 9);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 2, absx + 1, box_y + 8);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 2, absx + 9, box_y + 8);
	
	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	}
	
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, box_y + 2, absx, box_y + 8);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y, absx + 8, box_y);
	
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 1);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 1);
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, box_y + 9);
	
	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	}
	
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 10, absx + 8, box_y + 10);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 10, box_y + 2, absx + 10, box_y + 8);
	
	XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, box_y + 9);
	
	if (GEEDOT_CHECKBOX(widget->widget_data)->is_checked) {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid2.color.pixel);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 2);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 5);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 5, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 5, box_y + 5);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 2);
      
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark2.color.pixel);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 2);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 5, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 5);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 4);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 8);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 8, box_y + 7);
		
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, box_y + 7);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 3, box_y + 6);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 6, box_y + 3);
		XDrawPoint(widget->display, widget->backbuffer, widget->frame_gc, absx + 7, box_y + 2);
	}
	else {      
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 2, absx + 1, absy + 8);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 1, absx + 8, absy + 1);
		
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 9, absx + 8, absy + 9);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 9, absy + 2, absx + 9, absy + 8);
	}
	
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->background.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 12, absy, widget->area.width - 12, widget->area.height);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->foreground.color.pixel);
	XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 13, box_y + 10, GEEDOT_CHECKBOX(widget->widget_data)->label, strlen(GEEDOT_CHECKBOX(widget->widget_data)->label));
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_checkbox_set_depressed (GeedotWidget *widget) {
	*widget->pointer_state = BUTTON1_DEPRESSED;
	gdot_redraw_widget(widget);
}

int gdot_checkbox_set_selected (GeedotWidget *widget) {
	GEEDOT_CHECKBOX(widget->widget_data)->is_checked = !GEEDOT_CHECKBOX(widget->widget_data)->is_checked;
	*widget->pointer_state = 0;
	gdot_redraw_widget(widget);
}

int gdot_checkbox_set_label (GeedotWidget *widget, char *label) {
	int font_char_width = XTextWidth(widget->font, " ", 1);
	int font_char_height = widget->font->ascent + widget->font->descent;
	
	GEEDOT_CHECKBOX(widget->widget_data)->label = (char *) realloc(GEEDOT_CHECKBOX(widget->widget_data)->label, (strlen(label) + 1) * sizeof(char));
	strcpy(GEEDOT_CHECKBOX(widget->widget_data)->label, label);
	
	widget->area.width = font_char_width * strlen(label) + 13;
	widget->area.height = font_char_height;

	gdot_redraw_widget(widget);
}
