/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitcombobox.c - combo box creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Dec, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>

GeedotWidget *gdot_add_combobox (GeedotWidget *frame, GeedotArea area) {
}

GeedotWidget *gdot_add_locked_combobox (GeedotWidget *frame, GeedotArea area) {
	GeedotAction action_temp;
	GeedotArea   sub_area;
	GeedotWidget *last_widget;
	GeedotWidget *popup;
	int font_char_width = XTextWidth(frame->font, " ", 1);
	int font_char_height = frame->font->ascent + frame->font->descent;

	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->type = GEEDOT_COMBOBOX;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_combobox);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	if (area.width < 0) {
		frame->last_widget->widget->area.width = font_char_width;
	}
	else {
		frame->last_widget->widget->area.width = area.width;
	}
	if (area.height < 0) {
		frame->last_widget->widget->area.height = font_char_height;
	}
	else {
		frame->last_widget->widget->area.height = area.height;
	}

	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;

	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;

	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;

	frame->last_widget->widget->widget_data = (GeedotCombobox *) malloc(sizeof(GeedotCombobox));
	GEEDOT_COMBOBOX(frame->last_widget->widget->widget_data)->locked = True;
	last_widget = frame->last_widget->widget;
	sub_area.x = 0;
	sub_area.y = 0;
	sub_area.width = -1;
	sub_area.height = -1;
	GEEDOT_COMBOBOX(last_widget->widget_data)->button = gdot_add_button(last_widget, sub_area);
	sub_area.x = gdot_toplevel_widget(last_widget)->area.x + gdot_widget_absolute_x(last_widget);
	sub_area.y = gdot_toplevel_widget(last_widget)->area.y + gdot_widget_absolute_y(last_widget) + GEEDOT_COMBOBOX(last_widget->widget_data)->button->area.height;
	sub_area.width = 5;
	sub_area.height = 5;
	popup = GEEDOT_COMBOBOX(last_widget->widget_data)->popup = gdot_add_popup_frame(last_widget, sub_area);
	GEEDOT_COMBOBOX(frame->last_widget->widget->widget_data)->option = NULL;
	GEEDOT_COMBOBOX(frame->last_widget->widget->widget_data)->first_entry = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	GEEDOT_COMBOBOX(frame->last_widget->widget->widget_data)->first_entry->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	GEEDOT_COMBOBOX(frame->last_widget->widget->widget_data)->last_entry = GEEDOT_COMBOBOX(frame->last_widget->widget->widget_data)->first_entry;

	action_temp.trigger_mask = GeedotButtonPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_combobox_set_depressed);
	action_temp.response_data = last_widget;
	gdot_register_widget_action(GEEDOT_COMBOBOX(last_widget->widget_data)->button, action_temp);

	action_temp.trigger_mask = GeedotButtonRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_combobox_set_released);
	gdot_register_widget_action(GEEDOT_COMBOBOX(last_widget->widget_data)->button, action_temp);

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;

	gdot_redraw_widget(last_widget);

	return last_widget;
}

GeedotWidget *gdot_add_combobox_entry (GeedotWidget *combobox, char *text) {
	GeedotWidget *last_widget;
	GeedotWidget *button;
	GeedotWidgetList *entry_list;
	GeedotAction  action;
	GeedotArea area_;
	int font_char_width = XTextWidth(combobox->parent->font, " ", 1);
	int font_char_height = combobox->parent->font->ascent + combobox->parent->font->descent;
	int last_entry_bottom = 0;

	entry_list = GEEDOT_COMBOBOX(combobox->widget_data)->first_entry;
	while (entry_list != GEEDOT_COMBOBOX(combobox->widget_data)->last_entry) {
		last_entry_bottom += entry_list->widget->area.height;
		entry_list = entry_list->next;
	}
	area_.x = 0;
	area_.y = last_entry_bottom;
	area_.width = -1;
	area_.height = -1;
	button = gdot_add_button(GEEDOT_COMBOBOX(combobox->widget_data)->popup, area_);
	button->owner = combobox;
	button->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_combobox_button);

	GEEDOT_COMBOBOX(combobox->widget_data)->last_entry->widget = button;
	GEEDOT_COMBOBOX(combobox->widget_data)->last_entry->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	GEEDOT_COMBOBOX(combobox->widget_data)->last_entry->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	GEEDOT_COMBOBOX(combobox->widget_data)->last_entry = GEEDOT_COMBOBOX(combobox->widget_data)->last_entry->next;
	gdot_button_set_label(button, text);
	gdot_button_set_label(GEEDOT_COMBOBOX(combobox->widget_data)->button, text);

	action.trigger_mask = GeedotPointerMotion;
	action.response = GEEDOT_FUNCTION(gdot_widget_steal_motion_focus);
	action.response_data = button;
	gdot_register_widget_action(button, action);
	action.response = GEEDOT_FUNCTION(gdot_widget_grab_depression_focus);
	gdot_register_widget_action(button, action);

	action.trigger_mask = GeedotButtonRelease;
	action.response = GEEDOT_FUNCTION(gdot_combobox_set_option);
	action.response_data = button;
	gdot_register_widget_action(button, action);

	gdot_widget_shrinkwrap(GEEDOT_COMBOBOX(combobox->widget_data)->popup);
	gdot_combobox_resize_popup(combobox, GEEDOT_COMBOBOX(combobox->widget_data)->popup->area);
	gdot_redraw_button(button);
	gdot_redraw_button(GEEDOT_COMBOBOX(combobox->widget_data)->button);

	return button;
}

int gdot_combobox_set_option (GeedotWidget *widget) {
	gdot_button_set_label(GEEDOT_COMBOBOX(widget->owner->widget_data)->button, GEEDOT_BUTTON(widget->widget_data)->label);
	GEEDOT_COMBOBOX(widget->owner->widget_data)->option = widget;
}

GeedotWidget *gdot_combobox_get_option (GeedotWidget *widget) {
	if (GEEDOT_COMBOBOX(widget->widget_data)->option != NULL) {
		return GEEDOT_COMBOBOX(widget->widget_data)->option;
	}
	else {
		return 0;
	}
}

char *gdot_combobox_get_option_string (GeedotWidget *widget) {
	if (GEEDOT_COMBOBOX(widget->widget_data)->option != NULL) {
		return GEEDOT_BUTTON(GEEDOT_COMBOBOX(widget->widget_data)->option)->label;
	}
	else {
		return "[no option]";
	}
}

int gdot_redraw_combobox (GeedotWidget *widget) {
	gdot_redraw_widget(GEEDOT_COMBOBOX(widget->widget_data)->button);
	gdot_redraw_widget(GEEDOT_COMBOBOX(widget->widget_data)->popup);
}

int gdot_redraw_combobox_button (GeedotWidget *widget) {
	if (gdot_widget_has_motion_focus(widget) && widget->is_active) {
		gdot_draw_combobox_button_depressed(widget);
	}
	else {
		gdot_draw_combobox_button_plain(widget);
	}

	if (!widget->is_active) {
		gdot_shade_widget(widget);
	}
}

int gdot_draw_combobox_button_depressed (GeedotWidget *widget) {
	unsigned int absx = gdot_widget_absolute_x(widget);
	unsigned int absy = gdot_widget_absolute_y(widget);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy, absx + widget->area.width - 1, absy);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, absx + 1, absy + widget->area.height - 1);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + widget->area.height - 1, absx + widget->area.width - 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width - 1, absy + 1, absx + widget->area.width - 1, absy + widget->area.height - 2);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 1, widget->area.width - 3, widget->area.height - 2);
	XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy + 1, absx, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy + 1, absx + widget->area.width, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width, widget->area.height, absx, absy);
}

int gdot_draw_combobox_button_plain (GeedotWidget *widget) {
	unsigned int absx = gdot_widget_absolute_x(widget);
	unsigned int absy = gdot_widget_absolute_y(widget);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy, absx + widget->area.width - 1, absy);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, absx + 1, absy + widget->area.height - 1);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + widget->area.height - 1, absx + widget->area.width - 1, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width - 1, absy + 1, absx + widget->area.width - 1, absy + widget->area.height - 2);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 2, absy + 1, widget->area.width - 3, widget->area.height - 2);
	XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy, absx, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy, absx + widget->area.width, absy + widget->area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawString(widget->display, widget->backbuffer, widget->frame_gc, absx + 4, absy + widget->font->ascent + 4, GEEDOT_BUTTON(widget->widget_data)->label, strlen(GEEDOT_BUTTON(widget->widget_data)->label));
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width, widget->area.height, absx, absy);
}

int gdot_combobox_set_depressed (GeedotWidget *widget) {
	*widget->pointer_state = BUTTON1_DEPRESSED;
	if (!GEEDOT_COMBOBOX(widget->widget_data)->popup->is_visible) {
		gdot_combobox_expand(widget);
	}
}

int gdot_combobox_set_released (GeedotWidget *widget) {
}

int gdot_combobox_expand (GeedotWidget *widget) {
	GeedotArea area;
	GeedotWidget *toplevel = gdot_toplevel_widget(widget);
	GeedotWidgetList *entry_list;
	XEvent event_holder;
	char *name;

	area.x = gdot_toplevel_widget(widget)->area.x + gdot_widget_absolute_x(GEEDOT_COMBOBOX(widget->widget_data)->button);
	area.y = gdot_toplevel_widget(widget)->area.y + gdot_widget_absolute_y(GEEDOT_COMBOBOX(widget->widget_data)->button) + GEEDOT_COMBOBOX(widget->widget_data)->button->area.height + 1;
	gdot_move_frame(GEEDOT_COMBOBOX(widget->widget_data)->popup, area);
	gdot_show_frame(GEEDOT_COMBOBOX(widget->widget_data)->popup);
	gdot_raise_frame(GEEDOT_COMBOBOX(widget->widget_data)->popup);

	entry_list = GEEDOT_COMBOBOX(widget->widget_data)->first_entry;
	while (entry_list != GEEDOT_COMBOBOX(widget->widget_data)->last_entry) {
		gdot_redraw_widget(entry_list->widget);
		entry_list = entry_list->next;
	}

	XGrabPointer(widget->display, GEEDOT_COMBOBOX(widget->widget_data)->popup->drawable, False, PointerMotionMask | ButtonPressMask | ButtonReleaseMask, GrabModeAsync, GrabModeAsync, None, None, CurrentTime);
	do {
		gdot_wait_for_event(widget);
		event_holder = gdot_process_next_event(widget);
	} while (event_holder.type != ButtonRelease);
	XUngrabPointer(widget->display, CurrentTime);
	gdot_combobox_collapse(widget);
}

int gdot_combobox_collapse (GeedotWidget *widget) {
	gdot_hide_frame(GEEDOT_COMBOBOX(widget->widget_data)->popup);
	gdot_widget_clear_focus();
	gdot_widget_steal_focus(GEEDOT_COMBOBOX(widget->widget_data)->button);
	*widget->pointer_state = 0;
}

int gdot_combobox_resize_popup (GeedotWidget *widget, GeedotArea area) {
	GeedotWidgetList *widget_list = GEEDOT_COMBOBOX(widget->widget_data)->first_entry;

	gdot_resize_frame(GEEDOT_COMBOBOX(widget->widget_data)->popup, area);
	while (widget_list != GEEDOT_COMBOBOX(widget->widget_data)->last_entry) {
		widget_list->widget->area.width = area.width - 1;
		gdot_redraw_widget(widget_list->widget);
		widget_list = widget_list->next;
	}
}
