/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitframe.c - frame handling
 *   Author: Luke Crowthers
 *   Start Date: Aug, 2000
 *   Version: 0.1a
 *   License: GPL
 */

#include <gdtoolkit.h>

int gdot_redraw_frame (GeedotWidget *widget) {
	int width, height, buffer;

	if (widget->type == GEEDOT_TOPLEVEL_FRAME || widget->type == GEEDOT_POPUP_FRAME) {
		XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, 0, 0, widget->area.width, widget->area.height, 0, 0);
	}
}

int gdot_redraw_exposed_frame (GeedotWidget *widget, void *event_data) {
	int x = ((XEvent *) event_data)->xexpose.x;
	int y = ((XEvent *) event_data)->xexpose.y;
	int width = ((XEvent *) event_data)->xexpose.width;
	int height = ((XEvent *) event_data)->xexpose.height;

	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, x, y, width, height, x, y);
}

int gdot_show_frame (GeedotWidget *widget) {
	XEvent map_wait_event;
	Window root;
	int buf;

	XMapWindow(widget->display, widget->drawable);
	for (;;) {
		XNextEvent(widget->display, &map_wait_event);
		if (map_wait_event.type == MapNotify) {
			break;
		}
		if (map_wait_event.type == ConfigureNotify) {
			gdot_frame_handle_reconfiguration(widget, &map_wait_event);
		}
	}
	widget->is_visible = True;
	return 1;
}

int gdot_hide_frame (GeedotWidget *widget) {
	XEvent unmap_wait_event;

	XUnmapWindow(widget->display, widget->drawable);
	for (;;) {
		XNextEvent(widget->display, &unmap_wait_event);
		if (unmap_wait_event.type == UnmapNotify) {
			break;
		}
	}
	widget->is_visible = False;
	return 1;
}

int gdot_reset_pointer (GeedotWidget *widget) {
	Window window_holder;
	int pointx, pointy;
	unsigned int button_mask;

	XQueryPointer(widget->display, widget->drawable, &window_holder, 
		      &window_holder, &pointx, &pointy, &pointx, &pointy, &button_mask);
	*widget->pointer_state = 0;
	if (button_mask & Button1Mask) {
		*widget->pointer_state = *widget->pointer_state | BUTTON1_DEPRESSED;
	}
}

int gdot_frame_handle_reconfiguration (GeedotWidget *widget, void *event_data) {
	Pixmap buffer;
	int width, height, depth, holder;
	Window root;

	widget->area.x = ((XEvent *) event_data)->xconfigure.x;
	widget->area.y = ((XEvent *) event_data)->xconfigure.y;
	widget->area.width = ((XEvent *) event_data)->xconfigure.width;
	widget->area.height = ((XEvent *) event_data)->xconfigure.height;
	XGetGeometry(widget->display, widget->backbuffer, &root, &holder, &holder, &width, &height, &holder, &depth);
	if ((widget->area.width != width) || (widget->area.height != height)) {
		buffer = XCreatePixmap(widget->display, widget->drawable, width, height, depth);
		XCopyArea(widget->display, widget->backbuffer, buffer, widget->frame_gc, 0, 0, width, height, 0, 0);
		XFreePixmap(widget->display, widget->backbuffer);
		widget->backbuffer = XCreatePixmap(widget->display, widget->drawable, widget->area.width, widget->area.height, DefaultDepth(widget->display, DefaultScreen(widget->display)));
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->background.color.pixel);
		XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, 0, 0, widget->area.width, widget->area.height);
		XCopyArea(widget->display, buffer, widget->backbuffer, widget->frame_gc, 0, 0, width, height, 0, 0);
		XFreePixmap(widget->display, buffer);
		gdot_widget_reinherit_recursive(widget);
		gdot_redraw_widget(widget);
	}
}

int gdot_move_frame (GeedotWidget *widget, GeedotArea area) {
	XWindowChanges xwchanges;
	unsigned int valmask;

	valmask = CWX | CWY;
	xwchanges.x = area.x;
	xwchanges.y = area.y;
	XConfigureWindow(widget->display, widget->drawable, valmask, &xwchanges);
	widget->area.x = area.x;
	widget->area.y = area.y;
}

int gdot_resize_frame (GeedotWidget *widget, GeedotArea area) {
	XWindowChanges xwchanges;
	unsigned int valmask;

	valmask = CWWidth | CWHeight;
	xwchanges.width = area.width;
	xwchanges.height = area.height;
	XConfigureWindow(widget->display, widget->drawable, valmask, &xwchanges);
	widget->area.width = area.width;
	widget->area.height = area.height;
}

int gdot_force_resize_frame (GeedotWidget *widget, GeedotArea area) {
	widget->area.width = -1;
	widget->area.width = -1;

	gdot_resize_frame(widget, area);
}

int gdot_raise_frame (GeedotWidget *widget) {
	XRaiseWindow(widget->display, widget->drawable);
}

int gdot_frame_clear (GeedotWidget *widget) {
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->background.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, 0, 0, widget->area.width, widget->area.height);
	gdot_redraw_widget(widget);
}
