/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitmain.c - init. code for gdtoolkit, as well as
 *         global control functions and event handling.
 *   Author: Luke Crowthers
 *   Start Date: Aug, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>
#include <sys/timeb.h>

GeedotWidget *gdot_get_root_widget (Display *display) {
	if (__gdot_root_widget != NULL) {
		puts("already have a root widget");
		return __gdot_root_widget;
	}
	__gdot_root_widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	__gdot_root_widget->display = display;
	__gdot_root_widget->parent = NULL;
	__gdot_root_widget->is_active = False;
	__gdot_root_widget->is_visible = False;
	__gdot_root_widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_null_function);
	__gdot_root_widget->area.x = 0;
	__gdot_root_widget->area.y = 0;
	__gdot_root_widget->area.width = 0;
	__gdot_root_widget->area.height = 0;

	__gdot_root_widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	__gdot_root_widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	__gdot_root_widget->last_widget = __gdot_root_widget->first_widget;

	__gdot_root_widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	__gdot_root_widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	__gdot_root_widget->last_action = __gdot_root_widget->first_action;

	__gdot_root_widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	__gdot_root_widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	__gdot_root_widget->last_widget_action = __gdot_root_widget->first_widget_action;
	return __gdot_root_widget;
}

int gdot_register_action (GeedotWidget *widget, GeedotAction action) {
	widget->last_action->action->trigger_mask = action.trigger_mask;
	widget->last_action->action->response = action.response;
	widget->last_action->action->response_data = action.response_data;
	widget->last_action->action->area.x = action.area.x;
	widget->last_action->action->area.y = action.area.y;
	widget->last_action->action->area.width = action.area.width;
	widget->last_action->action->area.height = action.area.height;
	
	widget->last_action->next = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	widget->last_action->next->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	
	widget->last_action = widget->last_action->next;
	
	return 1;
}

int gdot_unregister_action (GeedotWidget *widget, GeedotAction *action) {
	/*TODO: gdot_unregister_action*/
}

int gdot_register_widget_action (GeedotWidget *widget, GeedotAction action) {
	widget->last_widget_action->action->trigger_mask = action.trigger_mask;
	widget->last_widget_action->action->response = action.response;
	widget->last_widget_action->action->response_data = action.response_data;
	widget->last_widget_action->action->area.x = widget->area.x;
	widget->last_widget_action->action->area.y = widget->area.y;
	widget->last_widget_action->action->area.width = widget->area.width;
	widget->last_widget_action->action->area.height = widget->area.height;
	widget->handled_events = widget->handled_events | action.trigger_mask;

	widget->last_widget_action->next = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	widget->last_widget_action->next->action = (GeedotAction *) malloc(sizeof(GeedotAction));

	widget->last_widget_action = widget->last_widget_action->next;

	return 1;
}

int gdot_unregister_widget_action (GeedotWidget *widget, GeedotAction *action) {
	/*TODO: gdot_unregister_widget_action*/
}

int gdot_area_bounds_location (GeedotArea boundary, int x, int y) {
	if ((x >= boundary.x && x <= boundary.x + boundary.width) && (y >= boundary.y && y <= boundary.y + boundary.height)) {
		return 1;
	}
	else {
		return 0;
	}
}

XEvent gdot_process_next_event (GeedotWidget *frame) {
	XEvent event, event_send, evtck;
	GeedotActionList *action_list;
	GeedotWidgetList *widget_list;
	GeedotEventMask   event_type;

	if (XPending(frame->display)) {
		XNextEvent(frame->display, &event);
		switch (event.type) {
		 case MotionNotify:
			gdot_widget_verify_motion_focus(event.xbutton.x, event.xbutton.y);
			break;
		 case SelectionClear:
			puts("gads, I'm not the selection owner anymore");
			break;
		 case SelectionRequest:
			puts("sending the selection to requestor");
			event_send.type = SelectionNotify;
			event_send.xselection.type = SelectionNotify;
			event_send.xselection.requestor = event.xselectionrequest.requestor;
			event_send.xselection.selection = XInternAtom(frame->display, "PRIMARY", False);
			event_send.xselection.target = XInternAtom(frame->display, "STRING", False);
			XChangeProperty(frame->display, event.xselectionrequest.requestor, event.xselectionrequest.property, XInternAtom(frame->display, "STRING", False), 8, PropModeReplace, frame->selection, strlen(frame->selection));
			event_send.xselection.property = event.xselectionrequest.property;
			event_send.xselection.time = CurrentTime;
			XSendEvent(frame->display, event.xselectionrequest.requestor, True, 0, &event_send);
			break;
		 case SelectionNotify:
			puts("erm, something happend, I'm not sure what it means");
			break;
		}

		gdot_widget_run_actions_for_event(frame, event);
		gdot_widget_run_actions_recursive(frame, event);

		return event; 
	}
	else {
		return (XEvent) 0;
	}
}

int gdot_get_event_type_mask (int event_type) {
	switch (event_type) {
	 case ButtonPress:
		return ButtonPressMask;
	 case ButtonRelease:
		return ButtonReleaseMask;
	 case MotionNotify:
		return PointerMotionMask;
	 case Expose:
		return ExposureMask;
	 case EnterNotify:
		return EnterWindowMask;
	 case LeaveNotify:
		return LeaveWindowMask;
	}
	return 0;
}

int gdot_wait_for_event (GeedotWidget *widget) {
	XEvent event;

	XPeekEvent(widget->display, &event);
}

int gdot_events_pending (GeedotWidget *widget) {
	return XPending(widget->display);
}
