/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitpopupframe.c - popup frame creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Feb, 2001
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>
#include <stdio.h>

GeedotWidget *gdot_add_popup_frame (GeedotWidget *parent, GeedotArea area) {
	GeedotWidget *frame_holder = parent->last_widget->widget;
	GeedotAction action_holder;
	int defaultxsize = 200;
	int defaultysize = 200;
	int count, county;
	XSetWindowAttributes window_attribs;
	XGCValues gc_vals;
	Atom window_protocols;

	frame_holder->display = parent->display;
	frame_holder->parent = NULL;
	frame_holder->type = GEEDOT_TOPLEVEL_FRAME;
	frame_holder->is_active = True;
	frame_holder->is_visible = False;
	frame_holder->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_frame);
	frame_holder->draw_package.redraw_data = frame_holder; 
	frame_holder->area.x = area.x;
	frame_holder->area.y = area.y;
	if (area.width < 0) {
		frame_holder->area.width = defaultxsize;
	}
	else {
		frame_holder->area.width = area.width;
	}
	if (area.height < 0) {
		frame_holder->area.height = defaultysize;
	}
	else {
		frame_holder->area.height = area.height;
	}
	frame_holder->pointer_state = (GeedotPointerState *) malloc(sizeof(GeedotPointerState));
	*frame_holder->pointer_state = 0;

	window_attribs.event_mask = StructureNotifyMask | SubstructureNotifyMask;
	window_attribs.background_pixel = WhitePixel(frame_holder->display, DefaultScreen(frame_holder->display));
	window_attribs.override_redirect = True;

	frame_holder->drawable = XCreateWindow(frame_holder->display, DefaultRootWindow(frame_holder->display), area.x, area.y,
					       frame_holder->area.width, frame_holder->area.height, 0,
					       CopyFromParent, InputOutput, CopyFromParent,
					       CWEventMask | CWBackPixel | CWOverrideRedirect, &window_attribs);

	frame_holder->backbuffer = XCreatePixmap(frame_holder->display, frame_holder->drawable, frame_holder->area.width, frame_holder->area.height, DefaultDepth(frame_holder->display, DefaultScreen(frame_holder->display)));

	XSelectInput(frame_holder->display, frame_holder->drawable, StructureNotifyMask | 
		     ButtonPressMask | ButtonReleaseMask | PointerMotionMask | 
		     ExposureMask | EnterWindowMask | LeaveWindowMask | KeyPressMask |
		     KeyReleaseMask | VisibilityChangeMask);

	frame_holder->selection = (char *) malloc(sizeof(char));
	*frame_holder->selection = 0;

	window_protocols = XInternAtom(frame_holder->display, "WM_DELETE_WINDOW", False);
	XSetWMProtocols(frame_holder->display, frame_holder->drawable, &window_protocols, 1);

	gc_vals.foreground = BlackPixel(frame_holder->display, DefaultScreen(frame_holder->display));
	gc_vals.background = WhitePixel(frame_holder->display, DefaultScreen(frame_holder->display));
	frame_holder->frame_gc = XCreateGC(frame_holder->display, frame_holder->drawable, GCForeground | GCBackground, &gc_vals);

	frame_holder->font = XLoadQueryFont(frame_holder->display, "-*-clean-medium-r-*-*-10-*-*-*-*-*-*");
	if (frame_holder->font == NULL) {
		frame_holder->font = XLoadQueryFont(frame_holder->display, "fixed");
		if (frame_holder->font == NULL) {
			fprintf(stderr, "Could not find any fonts to use\n");
			exit(0);
		}
	}
	XSetFont(frame_holder->display, frame_holder->frame_gc, frame_holder->font->fid);

	frame_holder->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame_holder->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame_holder->last_widget = frame_holder->first_widget;

	frame_holder->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame_holder->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame_holder->last_action = frame_holder->first_action;

	frame_holder->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame_holder->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame_holder->last_widget_action = frame_holder->first_widget_action;

	action_holder.trigger_mask = GeedotFrameConfigure;
	action_holder.response = GEEDOT_FUNCTION(gdot_frame_handle_reconfiguration);
	action_holder.response_data = frame_holder;
	gdot_register_widget_action(frame_holder, action_holder);

	action_holder.trigger_mask = GeedotExpose;
	action_holder.response = GEEDOT_FUNCTION(gdot_redraw_exposed_frame);
	gdot_register_widget_action(frame_holder, action_holder);

	action_holder.trigger_mask = GeedotFrameLeave;
	action_holder.response = GEEDOT_FUNCTION(gdot_widget_clear_focus);
	action_holder.response_data = NULL;
	gdot_register_widget_action(frame_holder, action_holder);

	frame_holder->widget_data = (GeedotToplevelFrame *) malloc(sizeof(GeedotToplevelFrame));

	GEEDOT_TOPLEVEL_FRAME(frame_holder->widget_data)->title = (char *) malloc(17 * sizeof(char));
	strcpy(GEEDOT_TOPLEVEL_FRAME(frame_holder->widget_data)->title, "GeedotPopupFrame");
	XStoreName(frame_holder->display, frame_holder->drawable, GEEDOT_TOPLEVEL_FRAME(frame_holder->widget_data)->title);

	XSetForeground(frame_holder->display, frame_holder->frame_gc, WhitePixel(frame_holder->display, DefaultScreen(frame_holder->display)));
	XFillRectangle(frame_holder->display, frame_holder->backbuffer, frame_holder->frame_gc, 0, 0, frame_holder->area.width, frame_holder->area.height);

	XCopyArea(frame_holder->display, frame_holder->backbuffer, frame_holder->drawable, frame_holder->frame_gc, 0, 0, frame_holder->area.width, frame_holder->area.height, 0, 0);

	frame_holder->colormap = XCreateColormap(frame_holder->display, frame_holder->drawable, DefaultVisual(frame_holder->display, DefaultScreen(frame_holder->display)), AllocNone);

	frame_holder->geedot_colors = (GeedotDefaultColorSet *) malloc(sizeof(GeedotDefaultColorSet));
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:a8/a5/cc", &frame_holder->geedot_colors->light1.color, &frame_holder->geedot_colors->light1.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:57/52/7c", &frame_holder->geedot_colors->mid1.color, &frame_holder->geedot_colors->mid1.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:09/00/4f", &frame_holder->geedot_colors->dark1.color, &frame_holder->geedot_colors->dark1.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:ea/ea/ea", &frame_holder->geedot_colors->light2.color, &frame_holder->geedot_colors->light2.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:aa/aa/aa", &frame_holder->geedot_colors->mid2.color, &frame_holder->geedot_colors->mid2.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:8c/8c/8c", &frame_holder->geedot_colors->dark2.color, &frame_holder->geedot_colors->dark2.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:00/00/00", &frame_holder->geedot_colors->foreground.color, &frame_holder->geedot_colors->foreground.color);
	XAllocNamedColor(frame_holder->display, frame_holder->colormap, "rgb:ff/ff/ff", &frame_holder->geedot_colors->background.color, &frame_holder->geedot_colors->background.color);

	parent->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	parent->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	parent->last_widget = parent->last_widget->next;

	return frame_holder;
}
