/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitradio.c - radio button creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Sep, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>

int gdot_radio_button_grab_focus (GeedotWidget *);
int gdot_draw_radio_button_actuator (GeedotWidget *);

GeedotWidget *gdot_attach_radio_to_widget (GeedotWidget *attach_to) {
}

GeedotWidget *gdot_add_radio_list (GeedotWidget *frame, GeedotArea area) {
	GeedotWidget *last_widget;

	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->type = GEEDOT_RADIO_LIST;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	frame->last_widget->widget->area.width = -1;
	frame->last_widget->widget->area.height = -1;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_null_function);
	frame->last_widget->widget->draw_package.redraw_data = NULL;

	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;

	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;

	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;

	frame->last_widget->widget->widget_data = (GeedotRadioList *) malloc(sizeof(GeedotRadioList));
	GEEDOT_RADIO_LIST(frame->last_widget->widget->widget_data)->selected_button = NULL;

	last_widget = frame->last_widget->widget;

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;

	return last_widget;
}

GeedotWidget *gdot_add_radio_button (GeedotWidget *radio_list, GeedotArea area) {
	GeedotWidget *last_widget;
	GeedotAction  action_temp;
	int font_char_width = XTextWidth(radio_list->parent->font, " ", 1);
	int font_char_height = radio_list->font->ascent + radio_list->font->descent;
	GeedotWidget  *frame = radio_list;
	
	if (radio_list->type != GEEDOT_RADIO_LIST) {
		printf("GeedotRadioButtons may only be added to GeedotRadioLists\n");
		return NULL;
	}
	
	gdot_widget_inherit(radio_list, radio_list->last_widget->widget);
	radio_list->last_widget->widget->owner = radio_list;
	frame->last_widget->widget->type = GEEDOT_RADIO_BUTTON;
	frame->last_widget->widget->is_active = radio_list->is_active;
	frame->last_widget->widget->is_visible = radio_list->is_visible;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_radio_button);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	frame->last_widget->widget->area.width = XTextWidth(radio_list->font, "", strlen("")) + 14;
	frame->last_widget->widget->area.height = font_char_height;
	
	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;
	
	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;
	
	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;
	
	frame->last_widget->widget->widget_data = (GeedotRadioButton *) malloc(sizeof(GeedotRadioButton));
	GEEDOT_RADIO_BUTTON(frame->last_widget->widget->widget_data)->radio_list = radio_list->widget_data;
	GEEDOT_RADIO_BUTTON(frame->last_widget->widget->widget_data)->label = (char *) malloc(13 * sizeof(char));
	strcpy(GEEDOT_RADIO_BUTTON(frame->last_widget->widget->widget_data)->label, "");
	
	action_temp.trigger_mask = GeedotButtonRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_radio_button_set_selected);
	action_temp.response_data = frame->last_widget->widget;
	gdot_register_widget_action(frame->last_widget->widget, action_temp);
	
	last_widget = frame->last_widget->widget;
	
	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;
	
	gdot_redraw_widget(last_widget);
	
	return last_widget;
}

int gdot_radio_button_grab_focus (GeedotWidget *radio_button) {
	gdot_widget_grab_focus(radio_button);
	gdot_redraw_widget(radio_button);
}

int gdot_radio_button_set_selected (GeedotWidget *radio_button) {
	gdot_radio_button_set_deselected(radio_button->parent);
	GEEDOT_RADIO_LIST(radio_button->parent->widget_data)->selected_button = radio_button;
	gdot_redraw_widget(radio_button);
}

int gdot_radio_button_set_deselected (GeedotWidget *radio_list) {
	GeedotWidget *deselection = GEEDOT_RADIO_LIST(radio_list->widget_data)->selected_button;
	
	GEEDOT_RADIO_LIST(radio_list->widget_data)->selected_button = NULL;
	if (deselection != NULL) {
		gdot_redraw_widget(deselection);
	}
}

int gdot_radio_button_is_selected (GeedotWidget *radio_button) {
	if (radio_button == GEEDOT_RADIO_LIST(radio_button->parent->widget_data)->selected_button) {
		return 1;
	}
	else {
		return 0;
	}
}

int gdot_redraw_radio_button (GeedotWidget *radio_button) {
	if (gdot_radio_button_is_selected(radio_button)) {
		gdot_draw_radio_button_selected(radio_button);
	}
	else if (gdot_widget_has_motion_focus(radio_button) && radio_button->is_active && !(*radio_button->pointer_state & BUTTON1_DEPRESSED)) {
		gdot_draw_radio_button_highlighted(radio_button);
	}
	else {
		gdot_draw_radio_button_plain(radio_button);
	}
	
	if (!radio_button->is_active) {
		gdot_shade_widget(radio_button);
	}
}

int gdot_draw_radio_button_plain (GeedotWidget *radio_button) {
	int font_char_height = radio_button->font->ascent + radio_button->font->descent;
	unsigned int absx, absy;

	absx = gdot_widget_absolute_x(radio_button);
	absy = gdot_widget_absolute_y(radio_button);

	gdot_draw_radio_button_actuator(radio_button);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->background.color.pixel);
	XFillRectangle(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 12, absy, radio_button->area.width - 12, radio_button->area.height);
	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->foreground.color.pixel);
	XDrawString(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 13, absy + (font_char_height - 1), GEEDOT_RADIO_BUTTON(radio_button->widget_data)->label, strlen(GEEDOT_RADIO_BUTTON(radio_button->widget_data)->label));
	XCopyArea(radio_button->display, radio_button->backbuffer, radio_button->drawable, radio_button->frame_gc, absx, absy, radio_button->area.width + 1, radio_button->area.height + 1, absx, absy);
}

int gdot_draw_radio_button_highlighted (GeedotWidget *radio_button) {
	int font_char_height = radio_button->font->ascent + radio_button->font->descent;
	unsigned int absx, absy;

	absx = gdot_widget_absolute_x(radio_button);
	absy = gdot_widget_absolute_y(radio_button);

	gdot_draw_radio_button_actuator(radio_button);

	if (gdot_radio_button_is_selected(radio_button)) {
		gdot_draw_radio_button_actuator_selected(radio_button);
	}

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->light1.color.pixel);
	XFillRectangle(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 12, absy, radio_button->area.width - 12, radio_button->area.height);
	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->foreground.color.pixel);
	XDrawString(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 13, absy + (font_char_height - 1), GEEDOT_RADIO_BUTTON(radio_button->widget_data)->label, strlen(GEEDOT_RADIO_BUTTON(radio_button->widget_data)->label));
	XCopyArea(radio_button->display, radio_button->backbuffer, radio_button->drawable, radio_button->frame_gc, absx, absy, radio_button->area.width + 1, radio_button->area.height + 1, absx, absy);
}

int gdot_draw_radio_button_selected (GeedotWidget *radio_button) {
	if (gdot_widget_has_motion_focus(radio_button) && !(*radio_button->pointer_state & BUTTON1_DEPRESSED)) {
		gdot_draw_radio_button_highlighted(radio_button);
	}
	else {
		gdot_draw_radio_button_plain(radio_button);
	}
	gdot_draw_radio_button_actuator_selected(radio_button);
}

int gdot_draw_radio_button_actuator_selected (GeedotWidget *radio_button) {
	int y_start = gdot_widget_absolute_y(radio_button) + (radio_button->area.height) / 2 - 5;
	unsigned int absx, absy;

	absx = gdot_widget_absolute_x(radio_button);
	absy = gdot_widget_absolute_y(radio_button);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->light2.color.pixel);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start + 2, absx + 6, y_start + 2);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 3);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 7, y_start + 3);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 7);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 4, absx + 2, y_start + 6);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->mid2.color.pixel);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start + 3, absx + 6, y_start + 3);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 4, absx + 7, y_start + 4);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 5, absx + 7, y_start + 5);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 6, absx + 7, y_start + 6);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start + 7, absx + 6, y_start + 7);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->dark2.color.pixel);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 8, y_start + 4, absx + 8, y_start + 6);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start + 8, absx + 6, y_start + 8);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 7, y_start + 7);
	XCopyArea(radio_button->display, radio_button->backbuffer, radio_button->drawable, radio_button->frame_gc, absx, absy, radio_button->area.width + 1, radio_button->area.height + 1, absx, absy);
}

int gdot_draw_radio_button_actuator (GeedotWidget *radio_button) {
	int y_start = gdot_widget_absolute_y(radio_button) + (radio_button->area.height) / 2 - 5;
	unsigned int absx, absy;
	
	absx = gdot_widget_absolute_x(radio_button);
	absy = gdot_widget_absolute_y(radio_button);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->dark1.color.pixel);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 5, y_start);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 6, y_start);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 1);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 1);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 7, y_start + 1);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 8, y_start + 1);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 2);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 9, y_start + 2);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 3);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx, y_start + 4);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx, y_start + 5);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx, y_start + 6);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 7);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 8);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 9);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->mid1.color.pixel);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start + 1, absx + 6, y_start + 1);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 2, absx + 8, y_start + 2);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 3, absx + 9, y_start + 3);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 4, absx + 9, y_start + 4);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 5, absx + 9, y_start + 5);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 1, y_start + 6, absx + 9, y_start + 6);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 7, absx + 8, y_start + 7);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 2, y_start + 8, absx + 8, y_start + 8);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 3, y_start + 9, absx + 6, y_start + 9);

	XSetForeground(radio_button->display, radio_button->frame_gc, radio_button->geedot_colors->light1.color.pixel);
	XDrawLine(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 10, y_start + 4, absx + 10, y_start + 6);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 9, y_start + 7);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 9, y_start + 8);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 8, y_start + 9);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 7, y_start + 9);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 6, y_start + 10);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 5, y_start + 10);
	XDrawPoint(radio_button->display, radio_button->backbuffer, radio_button->frame_gc, absx + 4, y_start + 10);
	XCopyArea(radio_button->display, radio_button->backbuffer, radio_button->drawable, radio_button->frame_gc, absx, absy, radio_button->area.width + 1, radio_button->area.height + 1, absx, absy);
}

int gdot_radio_button_set_label (GeedotWidget *widget, char *label) {
	int font_char_height = widget->font->ascent + widget->font->descent;
	
	GEEDOT_RADIO_BUTTON(widget->widget_data)->label = (char *) realloc(GEEDOT_RADIO_BUTTON(widget->widget_data)->label, (strlen(label) + 1) * sizeof(char));
	strcpy(GEEDOT_RADIO_BUTTON(widget->widget_data)->label, label);
	
	widget->area.width = XTextWidth(widget->font, label, strlen(label)) + 14;
	widget->area.height = font_char_height;
	
	gdot_redraw_widget(widget);
}
