/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitscrollbar.c - scrollbar creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Nov, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>
#include <unistd.h>

GeedotWidget *gdot_add_scrollbar (GeedotWidget *frame, GeedotArea area) {
	GeedotAction  action_temp;
	GeedotArea    area_temp;
	GeedotWidget *last_widget;
	GeedotWidget *button_holder;

	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_scrollbar);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	if (area.width < 0) {
		frame->last_widget->widget->type = GEEDOT_VERT_SCROLLBAR;
		frame->last_widget->widget->area.width = 14;
		frame->last_widget->widget->area.height = area.height;
	}
	else if (area.height < 0) {
		frame->last_widget->widget->type = GEEDOT_HORIZ_SCROLLBAR;
		frame->last_widget->widget->area.height = 14;
		frame->last_widget->widget->area.width = area.width;
	}
	else {
		return NULL;
	}

	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;

	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;

	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;

	frame->last_widget->widget->widget_data = (GeedotScrollbar *) malloc(sizeof(GeedotScrollbar));
	GEEDOT_SCROLLBAR(frame->last_widget->widget->widget_data)->percent_start = 0.0;
	GEEDOT_SCROLLBAR(frame->last_widget->widget->widget_data)->percent = 100.0;
	GEEDOT_SCROLLBAR(frame->last_widget->widget->widget_data)->grab_start = -1;

	action_temp.trigger_mask = GeedotButtonPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_scrollbar_set_grab_start);
	action_temp.response_data = frame->last_widget->widget;
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	action_temp.trigger_mask = GeedotButtonRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_scrollbar_set_grab_stop);
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	/*action_temp.trigger_mask = GeedotPointerMotion;
	 action_temp.response = GEEDOT_FUNCTION(gdot_scrollbar_slide_bar);
	 gdot_register_widget_action(frame->last_widget->widget, action_temp);*/

	last_widget = frame->last_widget->widget;

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;

	area_temp.x = 0;
	area_temp.y = 0;
	area_temp.width = 14;
	area_temp.height = 14;
	GEEDOT_SCROLLBAR(last_widget->widget_data)->decrease_button = gdot_add_button(last_widget, area_temp);
	gdot_redraw_widget(GEEDOT_SCROLLBAR(last_widget->widget_data)->decrease_button);

	area_temp.x = last_widget->area.width - 14;
	area_temp.y = last_widget->area.height - 14;
	area_temp.width = 14;
	area_temp.height = 14;
	GEEDOT_SCROLLBAR(last_widget->widget_data)->increase_button = gdot_add_button(last_widget, area_temp);
	gdot_redraw_widget(GEEDOT_SCROLLBAR(last_widget->widget_data)->increase_button);

	action_temp.trigger_mask = GeedotButtonPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_scrollbar_decrement_bar);
	action_temp.response_data = last_widget;
	gdot_register_widget_action(GEEDOT_SCROLLBAR(last_widget->widget_data)->decrease_button, action_temp);

	action_temp.response = GEEDOT_FUNCTION(gdot_scrollbar_increment_bar);
	gdot_register_widget_action(GEEDOT_SCROLLBAR(last_widget->widget_data)->increase_button, action_temp);

	gdot_redraw_widget(last_widget);

	return last_widget;
}

int gdot_redraw_scrollbar (GeedotWidget *widget) {
	int north, south, east, west;

	north = gdot_widget_absolute_y(widget);
	south = north + widget->area.height;
	west  = gdot_widget_absolute_x(widget);
	east  = west + widget->area.width;

	if (widget->type == GEEDOT_VERT_SCROLLBAR) {
		north += 15;
		south -= 15;
		XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west, north - 1, west, south + 1);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, east, north - 1, east, south + 1);
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark2.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west + 1, north, east - 1, north);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west + 1, north + 1, west + 1, south);
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west + 2, south, east - 1, south);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, east - 1, north + 1, east - 1, south);
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid2.color.pixel);
		XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, west + 2, north + 1, east - west - 3, south - north - 1);
	}
	else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
		west += 15;
		east -= 15;
		XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west - 1, north, east + 1, north);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west - 1, south, east + 1, south);
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark2.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west, north + 1, east, north + 1);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west, north + 2, west, south - 2);
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, west, south - 1, east, south - 1);
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, east, north + 2, east, south - 2);
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid2.color.pixel);
		XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, west + 1, north + 2, east - west - 1, south - north - 3);
	}
	gdot_draw_scrollbar_plain(widget);

	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, widget->area.x, widget->area.y, widget->area.width + 1, widget->area.height + 1, widget->area.x, widget->area.y);
}

int gdot_draw_scrollbar_plain (GeedotWidget *widget) {
	GeedotArea grab_bar_area;

	if (widget->type == GEEDOT_VERT_SCROLLBAR) {
		grab_bar_area.x = gdot_widget_absolute_x(widget) + 2;
		grab_bar_area.y = (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start) / 100 * (widget->area.height - 32) + 16;
		grab_bar_area.width = widget->area.width - 4;
		grab_bar_area.height = (GEEDOT_SCROLLBAR(widget->widget_data)->percent) / 100 * (widget->area.height - 32);
	}
	else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
		grab_bar_area.x = (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start) / 100 * (widget->area.width - 32) + 16;
		grab_bar_area.y = gdot_widget_absolute_y(widget) + 2;
		grab_bar_area.width = (GEEDOT_SCROLLBAR(widget->widget_data)->percent) / 100 * (widget->area.width - 32);
		grab_bar_area.height = widget->area.height - 4;
	}

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->mid1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + 1, grab_bar_area.y, grab_bar_area.x + grab_bar_area.width - 1, grab_bar_area.y);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x, grab_bar_area.y + 1, grab_bar_area.x, grab_bar_area.y + grab_bar_area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + grab_bar_area.width - 1, grab_bar_area.y + 2, grab_bar_area.x + grab_bar_area.width - 1, grab_bar_area.y + grab_bar_area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + 2, grab_bar_area.y + grab_bar_area.height - 1, grab_bar_area.x + grab_bar_area.width - 2, grab_bar_area.y + grab_bar_area.height - 1);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light2.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + 1, grab_bar_area.y + 1, grab_bar_area.x + grab_bar_area.width - 1, grab_bar_area.y + 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + 1, grab_bar_area.y + 2, grab_bar_area.x + 1, grab_bar_area.y + grab_bar_area.height - 1);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + grab_bar_area.width, grab_bar_area.y + 1, grab_bar_area.x + grab_bar_area.width, grab_bar_area.y + grab_bar_area.height - 1);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + 1, grab_bar_area.y + grab_bar_area.height, grab_bar_area.x + grab_bar_area.width - 1, grab_bar_area.y + grab_bar_area.height);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, grab_bar_area.x + 2, grab_bar_area.y + 2, grab_bar_area.width - 3, grab_bar_area.height - 3);
}

int gdot_scrollbar_set_grab_start (GeedotWidget *widget, void *event_data) {
	XEvent evt;
	float  px_per_pr = 0.0;

	if (widget->type == GEEDOT_VERT_SCROLLBAR) {
		px_per_pr = (widget->area.height - 32) / 100.0;
		GEEDOT_SCROLLBAR(widget->widget_data)->grab_start = ((XEvent *) event_data)->xbutton.y - (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start * px_per_pr);
	}
	else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
		px_per_pr = (widget->area.width - 32) / 100.0;
		GEEDOT_SCROLLBAR(widget->widget_data)->grab_start = ((XEvent *) event_data)->xbutton.x - (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start * px_per_pr);
	}

	if (GEEDOT_SCROLLBAR(widget->widget_data)->grab_start >= 16 && 
	    GEEDOT_SCROLLBAR(widget->widget_data)->grab_start <=
	    (GEEDOT_SCROLLBAR(widget->widget_data)->percent) * px_per_pr + 16) {
		do {
			XNextEvent(widget->display, &evt);
			if (evt.type == MotionNotify) {
				gdot_scrollbar_slide_bar(widget, &evt);
			}
		} while (evt.type != ButtonRelease);
		gdot_scrollbar_set_grab_stop(widget, &evt);
	}
}

int gdot_scrollbar_slide_bar (GeedotWidget *widget, void *event_data) {
	float percent_per_pixel = 0.0;
	XEvent event = *((XEvent *) event_data);

	if (GEEDOT_SCROLLBAR(widget->widget_data)->grab_start >= 0) {
		if (widget->type == GEEDOT_VERT_SCROLLBAR) {
			GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = (((XEvent *) event_data)->xbutton.y - GEEDOT_SCROLLBAR(widget->widget_data)->grab_start) * (100.0 / (widget->area.height - 32));
		}
		else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
			GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = (((XEvent *) event_data)->xbutton.x - GEEDOT_SCROLLBAR(widget->widget_data)->grab_start) * (100.0 / (widget->area.width - 32));
		}
		if ((GEEDOT_SCROLLBAR(widget->widget_data)->percent_start + GEEDOT_SCROLLBAR(widget->widget_data)->percent) > 100.0) {
			GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = 100 - GEEDOT_SCROLLBAR(widget->widget_data)->percent;
		}
		if (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start < 0) {
			GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = 0;
		}
		gdot_redraw_widget(widget);
	}
}

int gdot_scrollbar_set_grab_stop (GeedotWidget *widget, void *event_data) {
	GEEDOT_SCROLLBAR(widget->widget_data)->grab_start = -1;
}

int gdot_scrollbar_decrement_bar (GeedotWidget *widget, void *event_data) {
	float percent_per_pixel;
	XEvent event = *((XEvent *) event_data);

	if (widget->type == GEEDOT_VERT_SCROLLBAR) {
		percent_per_pixel = 100 / (widget->area.height - 32);
	}
	else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
		percent_per_pixel = 100 / (widget->area.width - 32);
	}
	GEEDOT_SCROLLBAR(widget->widget_data)->percent_start -= 0.5;
	if (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start < 0) {
		GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = 0;
	}
	else {
		gdot_redraw_widget(widget);
		usleep(100000);
	}
	if (XPending(widget->display)) {
		XPeekEvent(widget->display, &event);
		if (event.type != ButtonRelease) {
			XNextEvent(widget->display, &event);
		}
	}

	while (event.type != ButtonRelease) {
		if (widget->type == GEEDOT_VERT_SCROLLBAR) {
			percent_per_pixel = 100 / (widget->area.height - 32);
		}
		else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
			percent_per_pixel = 100 / (widget->area.width - 32);
		}
		GEEDOT_SCROLLBAR(widget->widget_data)->percent_start -= 0.5;
		if (GEEDOT_SCROLLBAR(widget->widget_data)->percent_start < 0) {
			GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = 0;
		}
		else {
			gdot_redraw_widget(widget);
			usleep(150);
		}
		if (XCheckTypedEvent(widget->display, ButtonRelease, &event)) {
			XPutBackEvent(widget->display, &event);
			return 0;
		}
	}
}

int gdot_scrollbar_increment_bar (GeedotWidget *widget, void *event_data) {
	float percent_per_pixel;
	XEvent event = *((XEvent *) event_data);

	if (widget->type == GEEDOT_VERT_SCROLLBAR) {
		percent_per_pixel = 100 / (widget->area.height - 32);
	}
	else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
		percent_per_pixel = 100 / (widget->area.width - 32);
	}
	GEEDOT_SCROLLBAR(widget->widget_data)->percent_start += 0.5;
	if ((GEEDOT_SCROLLBAR(widget->widget_data)->percent_start + GEEDOT_SCROLLBAR(widget->widget_data)->percent) > 100) {
		GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = 100 - GEEDOT_SCROLLBAR(widget->widget_data)->percent;
	}
	else {
		gdot_redraw_widget(widget);
		usleep(100000);
	}
	if (XPending(widget->display)) {
		XPeekEvent(widget->display, &event);
		if (event.type != ButtonRelease) {
			XNextEvent(widget->display, &event);
		}
	}

	while (event.type != ButtonRelease) {
		if (widget->type == GEEDOT_VERT_SCROLLBAR) {
			percent_per_pixel = 100 / (widget->area.height - 32);
		}
		else if (widget->type == GEEDOT_HORIZ_SCROLLBAR) {
			percent_per_pixel = 100 / (widget->area.width - 32);
		}
		GEEDOT_SCROLLBAR(widget->widget_data)->percent_start += 0.5;
		if ((GEEDOT_SCROLLBAR(widget->widget_data)->percent_start + GEEDOT_SCROLLBAR(widget->widget_data)->percent) > 100) {
			GEEDOT_SCROLLBAR(widget->widget_data)->percent_start = 100 - GEEDOT_SCROLLBAR(widget->widget_data)->percent;
		}
		else {
			gdot_redraw_widget(widget);
			usleep(150);
		}
		if (XCheckTypedEvent(widget->display, ButtonRelease, &event)) {
			XPutBackEvent(widget->display, &event);
			return 0;
		}
	}
}

int gdot_scrollbar_set_percentage (GeedotWidget *widget, float percentage) {
	GEEDOT_SCROLLBAR(widget->widget_data)->percent = percentage;
}
