/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkittext.c - text creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Jan, 2001
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>

GeedotWidget *gdot_add_text (GeedotWidget *frame, GeedotArea area, char *text) {
	GeedotAction  action_temp;
	GeedotWidget *last_widget;
	int font_char_width = XTextWidth(frame->font, " ", 1);
	int font_char_height = frame->font->ascent + frame->font->descent;

	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->type = GEEDOT_TEXT;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_text);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	frame->last_widget->widget->area.width = XTextWidth(frame->font, text, strlen(text));
	frame->last_widget->widget->area.height = font_char_height;

	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;

	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;

	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;

	frame->last_widget->widget->widget_data = (GeedotText *) malloc(sizeof(GeedotText));
	GEEDOT_TEXT(frame->last_widget->widget->widget_data)->text = (char *) malloc((strlen(text) + 1) * sizeof(char));
	strcpy(GEEDOT_TEXT(frame->last_widget->widget->widget_data)->text, text);

	last_widget = frame->last_widget->widget;

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;

	return last_widget;
}

int gdot_redraw_text (GeedotWidget *widget) {
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->foreground.color.pixel);
	XDrawString(widget->display, widget->backbuffer, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget) + widget->font->ascent, GEEDOT_TEXT(widget->widget_data)->text, strlen(GEEDOT_TEXT(widget->widget_data)->text));
	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget), widget->area.width, widget->area.height, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget));
}

int gdot_redraw_label_text (GeedotWidget *widget) {
	if (gdot_widget_has_motion_focus(widget->parent) && widget->parent->is_active) {
		if (gdot_widget_has_focus(widget->parent)) {
			if (*widget->pointer_state & BUTTON1_DEPRESSED) {
				XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
				XDrawString(widget->display, widget->backbuffer, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget) + widget->font->ascent, GEEDOT_TEXT(widget->widget_data)->text, strlen(GEEDOT_TEXT(widget->widget_data)->text));
				XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget), widget->area.width, widget->area.height, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget));
			}
			else {
				XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
				XDrawString(widget->display, widget->backbuffer, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget) + widget->font->ascent, GEEDOT_TEXT(widget->widget_data)->text, strlen(GEEDOT_TEXT(widget->widget_data)->text));
				XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget), widget->area.width, widget->area.height, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget));
			}
		}
		else {
			if (!(*widget->pointer_state & BUTTON1_DEPRESSED)) {
				XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
				XDrawString(widget->display, widget->backbuffer, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget) + widget->font->ascent, GEEDOT_TEXT(widget->widget_data)->text, strlen(GEEDOT_TEXT(widget->widget_data)->text));
				XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget), widget->area.width, widget->area.height, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget));
			}
		}
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
		XDrawString(widget->display, widget->backbuffer, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget) + widget->font->ascent, GEEDOT_TEXT(widget->widget_data)->text, strlen(GEEDOT_TEXT(widget->widget_data)->text));
		XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget), widget->area.width, widget->area.height, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget));
	}
	if (!widget->parent->is_active) {
		gdot_shade_widget(widget);
	}
}

int gdot_text_set (GeedotWidget *widget, char *text) {
	GEEDOT_TEXT(widget->widget_data)->text = (char *) realloc(GEEDOT_TEXT(widget->widget_data)->text, (strlen(text) + 1) * sizeof(char));
	strcpy(GEEDOT_TEXT(widget->widget_data)->text, text);
	gdot_redraw_widget(widget);
}

int gdot_text_set_as_label (GeedotWidget *widget) {
	widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_label_text);
	widget->is_active = False;
}
