/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkittextbox.c - textbox creation, drawing, and handling
 *   Author: Luke Crowthers
 *   Start Date: Nov, 2000
 *              Modified for new GeedotWidget handling methods: Jan, 2001
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>
#include <X11/keysym.h>

GeedotWidget *gdot_add_textbox (GeedotWidget *frame, GeedotArea area) {
	GeedotAction  action_temp;
	GeedotWidget *last_widget;
	int font_width = XTextWidth(frame->font, " ", 1);
	int font_height = frame->font->ascent + frame->font->descent;

	gdot_widget_inherit(frame, frame->last_widget->widget);
	frame->last_widget->widget->owner = frame;
	frame->last_widget->widget->type = GEEDOT_TEXTBOX;
	frame->last_widget->widget->is_active = True;
	frame->last_widget->widget->is_visible = True;
	frame->last_widget->widget->draw_package.redraw = GEEDOT_FUNCTION(gdot_redraw_textbox);
	frame->last_widget->widget->draw_package.redraw_data = frame->last_widget->widget;
	frame->last_widget->widget->area.x = area.x;
	frame->last_widget->widget->area.y = area.y;
	frame->last_widget->widget->area.width = area.width;
	frame->last_widget->widget->area.height = font_height + 6;

	frame->last_widget->widget->first_widget = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->widget->first_widget->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget->widget->last_widget = frame->last_widget->widget->first_widget;

	frame->last_widget->widget->first_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_action = frame->last_widget->widget->first_action;

	frame->last_widget->widget->first_widget_action = (GeedotActionList *) malloc(sizeof(GeedotActionList));
	frame->last_widget->widget->first_widget_action->action = (GeedotAction *) malloc(sizeof(GeedotAction));
	frame->last_widget->widget->last_widget_action = frame->last_widget->widget->first_widget_action;

	frame->last_widget->widget->widget_data = (GeedotTextbox *) malloc(sizeof(GeedotTextbox));
	GEEDOT_TEXTBOX(frame->last_widget->widget->widget_data)->cursor_start = 2;
	GEEDOT_TEXTBOX(frame->last_widget->widget->widget_data)->cursor_stop  = 2;
	GEEDOT_TEXTBOX(frame->last_widget->widget->widget_data)->view_start   = 0;
	GEEDOT_TEXTBOX(frame->last_widget->widget->widget_data)->label = (char *) malloc(strlen("textbox") * sizeof(char));
	GEEDOT_TEXTBOX(frame->last_widget->widget->widget_data)->text_pixmap = XCreatePixmap(frame->display, frame->drawable, 1, 1, DefaultDepth(frame->display, DefaultScreen(frame->display)));
	strcpy(GEEDOT_TEXTBOX(frame->last_widget->widget->widget_data)->label, "textbox");

	action_temp.trigger_mask = GeedotButtonPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_textbox_set_cursor_start);
	action_temp.response_data = frame->last_widget->widget;
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	action_temp.trigger_mask = GeedotButtonRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_textbox_set_cursor_stop);
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	action_temp.trigger_mask = GeedotKeyPress;
	action_temp.response = GEEDOT_FUNCTION(gdot_textbox_handle_keypress);
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	action_temp.trigger_mask = GeedotKeyRelease;
	action_temp.response = GEEDOT_FUNCTION(gdot_textbox_handle_keyrelease);
	gdot_register_widget_action(frame->last_widget->widget, action_temp);

	last_widget = frame->last_widget->widget;

	frame->last_widget->next = (GeedotWidgetList *) malloc(sizeof(GeedotWidgetList));
	frame->last_widget->next->widget = (GeedotWidget *) malloc(sizeof(GeedotWidget));
	frame->last_widget = frame->last_widget->next;

	gdot_redraw_widget(last_widget);

	return last_widget;
}

int gdot_redraw_textbox (GeedotWidget *widget) {
	int char_width = XTextWidth(widget->font, " ", 1);
	int pmwidth, pmheight;
	unsigned int absx, absy;
	Window root_holder;

	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->dark1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy, absx + widget->area.width, absy);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx, absy + 1, absx, absy + widget->area.height);
	XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + widget->area.height, absx + widget->area.width - 1, absy + widget->area.height);
	XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + widget->area.width, absy + 1, absx + widget->area.width, absy + widget->area.height);

	XSetForeground(widget->display, widget->frame_gc, WhitePixel(widget->display, DefaultScreen(widget->display)));
	XFillRectangle(widget->display, widget->backbuffer, widget->frame_gc, absx + 1, absy + 1, widget->area.width - 1, widget->area.height - 1);
	if (GEEDOT_TEXTBOX(widget->widget_data)->text != NULL) {
		XGetGeometry(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, &root_holder, &char_width, &char_width, &pmwidth, &pmheight, &char_width, &char_width);
		XCopyArea(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->backbuffer, widget->frame_gc, GEEDOT_TEXTBOX(widget->widget_data)->view_start, 0, widget->area.width - 2, pmheight, absx + 2, absy + 4);
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start != GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop) {
		if (widget->is_active) {
			XSetForeground(widget->display, widget->frame_gc, widget->geedot_colors->light1.color.pixel);
		}

		if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop > GEEDOT_TEXTBOX(widget->widget_data)->cursor_start) {
			XFillRectangle(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - 2, 0, GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - GEEDOT_TEXTBOX(widget->widget_data)->cursor_start, widget->font->ascent + widget->font->descent);
			XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
			XDrawString(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - 2, widget->font->ascent, GEEDOT_TEXTBOX(widget->widget_data)->selection, strlen(GEEDOT_TEXTBOX(widget->widget_data)->selection));
		}
		else {
			XFillRectangle(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - 2, 0, GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop, widget->font->ascent + widget->font->descent);
			XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
			XDrawString(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - 2, widget->font->ascent, GEEDOT_TEXTBOX(widget->widget_data)->selection, strlen(GEEDOT_TEXTBOX(widget->widget_data)->selection));
		}
		XCopyArea(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->backbuffer, widget->frame_gc, GEEDOT_TEXTBOX(widget->widget_data)->view_start, 0, widget->area.width - 2, pmheight, absx + 2, absy + 4);
	}
	else {
		XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
		XDrawLine(widget->display, widget->backbuffer, widget->frame_gc, absx + GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - GEEDOT_TEXTBOX(widget->widget_data)->view_start, absy + 4, absx + GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - GEEDOT_TEXTBOX(widget->widget_data)->view_start, absy + 3 + widget->font->ascent + widget->font->descent);
	}

	if (!widget->is_active) {
		gdot_shade_widget(widget);
	}

	XCopyArea(widget->display, widget->backbuffer, widget->drawable, widget->frame_gc, absx, absy, widget->area.width + 1, widget->area.height + 1, absx, absy);
}

int gdot_textbox_set_cursor_start (GeedotWidget *widget, void *event_data) {
	int char_width = XTextWidth(widget->font, " ", 1);
	int start_offset = 2;
	int x = ((XEvent *) event_data)->xbutton.x;
	int y = ((XEvent *) event_data)->xbutton.y;
	int pmwidth, pmheight, intbuffer;
	unsigned int absx, absy;
	Window rootbuffer;
	XEvent evt;

	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	if (x > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + absx - start_offset) {
		x = strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + absx + start_offset;
	}

	GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = (x - absx + GEEDOT_TEXTBOX(widget->widget_data)->view_start) / char_width * char_width + start_offset;

	/*if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start < 2) {
	 GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = 2;
	 }*/

	/*if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start < GEEDOT_TEXTBOX(widget->widget_data)->view_start) {
	 GEEDOT_TEXTBOX(widget->widget_data)->view_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - 2;
	 }*/

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + 2) {
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->view_start > GEEDOT_TEXTBOX(widget->widget_data)->cursor_start) {
		GEEDOT_TEXTBOX(widget->widget_data)->view_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - 2 - char_width;
	}
   
	GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = GEEDOT_TEXTBOX(widget->widget_data)->cursor_start;

	XGetGeometry(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, &rootbuffer, &intbuffer, &intbuffer, &pmwidth, &pmheight, &intbuffer, &intbuffer);

	XSetForeground(widget->display, widget->frame_gc, WhitePixel(widget->display, DefaultScreen(widget->display)));
	XFillRectangle(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, 0, 0, pmwidth, pmheight);

	XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
	XDrawString(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, 0, widget->font->ascent, GEEDOT_TEXTBOX(widget->widget_data)->text, strlen(GEEDOT_TEXTBOX(widget->widget_data)->text));

	gdot_redraw_widget(widget);
	do {
		XNextEvent(widget->display, &evt);
		if (evt.type == MotionNotify) {
			gdot_textbox_grab_selection(widget, &evt);
		}
	} while (evt.type != ButtonRelease);
	gdot_textbox_set_cursor_stop(widget, &evt);
}

int gdot_textbox_set_cursor_stop (GeedotWidget *widget, void *event_data) {
	int char_width = XTextWidth(widget->font, " ", 1);
	int start_offset = 2;
	int x = ((XEvent *) event_data)->xbutton.x;
	int y = ((XEvent *) event_data)->xbutton.y;
	unsigned int absx, absy;

	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	if (x > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + absx + start_offset) {
		x = strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + absx + start_offset;
	}

	GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = (x - absx + GEEDOT_TEXTBOX(widget->widget_data)->view_start) / char_width * char_width + start_offset;

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < 2) {
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < GEEDOT_TEXTBOX(widget->widget_data)->view_start) {
		GEEDOT_TEXTBOX(widget->widget_data)->view_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + 2) {
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->view_start < GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - (widget->area.width - 4)) {
		GEEDOT_TEXTBOX(widget->widget_data)->view_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - (widget->area.width - 4);
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop != GEEDOT_TEXTBOX(widget->widget_data)->cursor_start) {
		gdot_textbox_set_selection(widget);
	}

	gdot_redraw_widget(widget);
}

int gdot_textbox_set_label (GeedotWidget *widget, char *label) {
}

int gdot_textbox_set_text (GeedotWidget *widget, char *text) {
	int str_width = XTextWidth(widget->font, text, strlen(text));
	int str_height = widget->font->ascent + widget->font->descent;
	int pixmapwidth, bufferint;
	unsigned int absx, absy;
	Window rootbuffer;

	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	GEEDOT_TEXTBOX(widget->widget_data)->text = (char *) realloc(GEEDOT_TEXTBOX(widget->widget_data)->text, (strlen(text) + 1) * sizeof(char));
	strcpy(GEEDOT_TEXTBOX(widget->widget_data)->text, text);

	XGetGeometry(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, &rootbuffer, &bufferint, &bufferint, &pixmapwidth, &bufferint, &bufferint, &bufferint);

	if (str_width < pixmapwidth) {
		str_width = pixmapwidth;
	}
	XFreePixmap(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap);
	GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap = XCreatePixmap(widget->display, widget->backbuffer, str_width + 1, str_height, DefaultDepth(widget->display, DefaultScreen(widget->display)));

	XSetForeground(widget->display, widget->frame_gc, WhitePixel(widget->display, DefaultScreen(widget->display)));
	XFillRectangle(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, 0, 0, str_width + 1, str_height);
	XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
	XDrawString(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, 0, widget->font->ascent, text, strlen(text));
}

int gdot_textbox_grab_selection (GeedotWidget *widget, void *event_data) {
	int char_width = XTextWidth(widget->font, " ", 1);
	int start_offset = 2;
	int x = ((XEvent *) event_data)->xmotion.x;
	int y = ((XEvent *) event_data)->xmotion.y;
	unsigned int absx, absy;

	absx = gdot_widget_absolute_x(widget);
	absy = gdot_widget_absolute_y(widget);

	if (x > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + absx + start_offset) {
		x = strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + absx + start_offset;
	}

	GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = (x - absx + GEEDOT_TEXTBOX(widget->widget_data)->view_start) / char_width * char_width + start_offset;

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < 2) {
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < GEEDOT_TEXTBOX(widget->widget_data)->view_start) {
		GEEDOT_TEXTBOX(widget->widget_data)->view_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + 2) {
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + 2;
	}

	if (GEEDOT_TEXTBOX(widget->widget_data)->view_start < GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - (widget->area.width - 4)) {
		GEEDOT_TEXTBOX(widget->widget_data)->view_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - (widget->area.width - 4);
	}

	gdot_textbox_set_selection(widget);

	gdot_redraw_widget(widget);
}

int gdot_textbox_set_selection (GeedotWidget *widget) {
	int char_width   = XTextWidth(widget->font, " ", 1);
	int strloc       = 0;
	int start_offset = 2;
	int str_width = XTextWidth(widget->font, GEEDOT_TEXTBOX(widget->widget_data)->text, strlen(GEEDOT_TEXTBOX(widget->widget_data)->text));
	int str_height = widget->font->ascent + widget->font->descent;
	int cursor_str_start = (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - start_offset) / char_width;
	int cursor_str_stop  = (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - start_offset) / char_width;
	char *selection;
	Display *display;

	if (cursor_str_start > cursor_str_stop) {
		cursor_str_start = cursor_str_stop;
		cursor_str_stop = GEEDOT_TEXTBOX(widget->widget_data)->cursor_start / char_width;
	}

	selection = (char *) malloc((cursor_str_stop - cursor_str_start + 1) * sizeof(char));

	for(strloc = cursor_str_start; strloc < cursor_str_stop; strloc++) {
		selection[strloc - cursor_str_start] = GEEDOT_TEXTBOX(widget->widget_data)->text[strloc];
	}
	selection[strloc - cursor_str_start] = '\0';

	GEEDOT_TEXTBOX(widget->widget_data)->selection = (char *) realloc(GEEDOT_TEXTBOX(widget->widget_data)->selection, (strlen(selection) + 1) * sizeof(char));

	strcpy(GEEDOT_TEXTBOX(widget->widget_data)->selection, selection);
	widget->selection = GEEDOT_TEXTBOX(widget->widget_data)->selection;

	XChangeProperty(widget->display, widget->drawable, XInternAtom(widget->display, "PRIMARY", 0), XInternAtom(widget->display, "STRING", 0), 8, PropModeReplace, GEEDOT_TEXTBOX(widget->widget_data)->selection, strlen(GEEDOT_TEXTBOX(widget->widget_data)->selection));
	XSetSelectionOwner(widget->display, XInternAtom(widget->display, "PRIMARY", 0), widget->drawable, CurrentTime);

	XSetForeground(widget->display, widget->frame_gc, WhitePixel(widget->display, DefaultScreen(widget->display)));
	XFillRectangle(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, 0, 0, str_width, str_height);
	XSetForeground(widget->display, widget->frame_gc, BlackPixel(widget->display, DefaultScreen(widget->display)));
	XDrawString(widget->display, GEEDOT_TEXTBOX(widget->widget_data)->text_pixmap, widget->frame_gc, 0, widget->font->ascent, GEEDOT_TEXTBOX(widget->widget_data)->text, strlen(GEEDOT_TEXTBOX(widget->widget_data)->text));
}

int gdot_textbox_handle_keypress (GeedotWidget *widget, void *event_data) {
	int   char_width = XTextWidth(widget->font, " ", 1);
	int   start_offset = 2;
	char *text;
	char *append;
	int   strloc;
	int   placement;
	KeySym keysym;

	placement = (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - start_offset) / char_width;

	if (((XEvent *) event_data)->xkey.state & ShiftMask) {
		keysym = XLookupKeysym(&((XEvent *) event_data)->xkey, 1);
	}
	else {
		keysym = XLookupKeysym(&((XEvent *) event_data)->xkey, 0);
	}

	if (keysym == NoSymbol) {
		return 0;
	}
	append = (char *) malloc((strlen(XKeysymToString(keysym)) + 1) * sizeof(char));
	strcpy(append, XKeysymToString(keysym));

	switch (keysym) {
	 case XK_grave:
		strcpy(append, "`");
		break;
	 case XK_asciitilde:
		strcpy(append, "~");
		break;
	 case XK_exclam:
		strcpy(append, "!");
		break;
	 case XK_at:
		strcpy(append, "@");
		break;
	 case XK_numbersign:
		strcpy(append, "#");
		break;
	 case XK_dollar:
		strcpy(append, "$");
		break;
	 case XK_percent:
		strcpy(append, "%");
		break;
	 case XK_asciicircum:
		strcpy(append, "^");
		break;
	 case XK_ampersand:
		strcpy(append, "&");
		break;
	 case XK_asterisk:
		strcpy(append, "*");
		break;
	 case XK_parenleft:
		strcpy(append, "(");
		break;
	 case XK_parenright:
		strcpy(append, ")");
		break;
	 case XK_minus:
		strcpy(append, "-");
		break;
	 case XK_underscore:
		strcpy(append, "_");
		break;
	 case XK_equal:
		strcpy(append, "=");
		break;
	 case XK_plus:
		strcpy(append, "+");
		break;
	 case XK_bracketleft:
		strcpy(append, "[");
		break;
	 case XK_bracketright:
		strcpy(append, "]");
		break;
	 case XK_braceleft:
		strcpy(append, "{");
		break;
	 case XK_braceright:
		strcpy(append, "}");
		break;
	 case XK_backslash:
		strcpy(append, "\\");
		break;
	 case XK_bar:
		strcpy(append, "|");
		break;
	 case XK_semicolon:
		strcpy(append, ";");
		break;
	 case XK_colon:
		strcpy(append, ":");
		break;
	 case XK_apostrophe:
		strcpy(append, "'");
		break;
	 case XK_quotedbl:
		strcpy(append, "\"");
		break;
	 case XK_comma:
		strcpy(append, ",");
		break;
	 case XK_less:
		strcpy(append, "<");
		break;
	 case XK_period:
		strcpy(append, ".");
		break;
	 case XK_greater:
		strcpy(append, ">");
		break;
	 case XK_slash:
		strcpy(append, "/");
		break;
	 case XK_question:
		strcpy(append, "?");
		break;
	 case XK_space:
		strcpy(append, " ");
		break;
	 case XK_BackSpace:
		if (*GEEDOT_TEXTBOX(widget->widget_data)->text == 0 || gdot_textbox_collapse_selection(widget) || placement == 0) {
			return 0;
		}
		text = (char *) malloc(strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * sizeof(char));
		memcpy(text, GEEDOT_TEXTBOX(widget->widget_data)->text, (placement - 1) * sizeof(char));
		for (strloc = placement; strloc < strlen(GEEDOT_TEXTBOX(widget->widget_data)->text); strloc++) {
			text[strloc - 1] = GEEDOT_TEXTBOX(widget->widget_data)->text[strloc];
		}
		text[strloc - 1] = '\0';

		gdot_textbox_set_text(widget, text);

		GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop -= char_width;
		if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < GEEDOT_TEXTBOX(widget->widget_data)->view_start + char_width) {
			if (GEEDOT_TEXTBOX(widget->widget_data)->view_start > char_width) {
				GEEDOT_TEXTBOX(widget->widget_data)->view_start -= char_width;
			}
			else {
				GEEDOT_TEXTBOX(widget->widget_data)->view_start = 0;
				GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = start_offset;
			}
		}
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop;

		gdot_redraw_widget(widget);

		free(text);
		return 0;
	 case XK_Delete:
		if (*GEEDOT_TEXTBOX(widget->widget_data)->text == 0 || gdot_textbox_collapse_selection(widget)) {
			return 0;
		}
		text = (char *) malloc(strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * sizeof(char));
		memcpy(text, GEEDOT_TEXTBOX(widget->widget_data)->text, placement * sizeof(char));
		for (strloc = placement; strloc < strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) - 1; strloc++) {
			text[strloc] = GEEDOT_TEXTBOX(widget->widget_data)->text[strloc + 1];
		}
		text[strloc] = '\0';

		gdot_textbox_set_text(widget, text);

		gdot_redraw_widget(widget);

		free(text);
		return 0;
	 case XK_Shift_L:
		return 0;
	 case XK_Shift_R:
		return 0;
	 case XK_Alt_L:
		return 0;
	 case XK_Alt_R:
		return 0;
	 case XK_Control_L:
		return 0;
	 case XK_Control_R:
		return 0;
	 case XK_F1:
		return 0;
	 case XK_F2:
		return 0;
	 case XK_F3:
		return 0;
	 case XK_F4:
		return 0;
	 case XK_F5:
		return 0;
	 case XK_F6:
		return 0;
	 case XK_F7:
		return 0;
	 case XK_F8:
		return 0;
	 case XK_F9:
		return 0;
	 case XK_F10:
		return 0;
	 case XK_F11:
		return 0;
	 case XK_F12:
		return 0;
	 case XK_Print:
		return 0;
	 case XK_Sys_Req:
		return 0;
	 case XK_Pause:
		return 0;
	 case XK_Break:
		return 0;
	 case XK_Home:
		return 0;
	 case XK_Prior:
		return 0;
	 case XK_Next:
		return 0;
	 case XK_End:
		return 0;
	 case XK_Caps_Lock:
		return 0;
	 case XK_Num_Lock:
		return 0;
	 case XK_Scroll_Lock:
		return 0;
	 case XK_Return:
		return 0;
	 case XK_Left:
		GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop -= char_width;
		if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < GEEDOT_TEXTBOX(widget->widget_data)->view_start + char_width) {
			if (GEEDOT_TEXTBOX(widget->widget_data)->view_start > char_width) {
				while (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < GEEDOT_TEXTBOX(widget->widget_data)->view_start + char_width) {
					GEEDOT_TEXTBOX(widget->widget_data)->view_start--;
				}
			}
			else {
				GEEDOT_TEXTBOX(widget->widget_data)->view_start = 0;
				GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = start_offset;
			}
		}
		if (((XEvent *) event_data)->xkey.state & ShiftMask) {
			gdot_textbox_set_selection(widget);
		}
		else {
			GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop;
			gdot_textbox_set_selection(widget);
		}
		gdot_redraw_widget(widget);
		return 0;
	 case XK_Up:
		/*TODO: add support for history perhaps?*/
		return 0;
	 case XK_Down:
		return 0;
	 case XK_Right:
		if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop < strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + start_offset) {
			GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop += char_width;
		}
		else {
			while (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop > strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) * char_width + start_offset) {
				GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop -= char_width;
			}
		}
		while (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop > widget->area.width + GEEDOT_TEXTBOX(widget->widget_data)->view_start - 4) {
			GEEDOT_TEXTBOX(widget->widget_data)->view_start++;
		}
		if (((XEvent *) event_data)->xkey.state & ShiftMask) {
			gdot_textbox_set_selection(widget);
		}
		else {
			GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop;
			gdot_textbox_set_selection(widget);
		}
		
		gdot_redraw_widget(widget);
		
		return 0;
	 default:
		gdot_textbox_collapse_selection(widget);
		break;
	}
	placement = (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - start_offset) / char_width;
	
	text = (char *) malloc((strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) + strlen(append) + 1) * sizeof(char));
	
	memcpy(text, GEEDOT_TEXTBOX(widget->widget_data)->text, placement * sizeof(char));
	
	for(strloc = 0; strloc < strlen(append); strloc++) {
		text[placement + strloc] = append[strloc];
	}
	for(strloc = placement; strloc < strlen(GEEDOT_TEXTBOX(widget->widget_data)->text); strloc++) {
		text[strloc + strlen(append)] = GEEDOT_TEXTBOX(widget->widget_data)->text[strloc];
	}
	text[(strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) + strlen(append)) * sizeof(char)] = '\0';
	
	gdot_textbox_set_text(widget, text);
	
	GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop += char_width * strlen(append);
	while (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop > widget->area.width + GEEDOT_TEXTBOX(widget->widget_data)->view_start - 4) {
		GEEDOT_TEXTBOX(widget->widget_data)->view_start++;
	}
	GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop;
	
	gdot_redraw_widget(widget);
	free(text);
}

int gdot_textbox_handle_keyrelease (GeedotWidget *widget, void *event_data) {
}

int gdot_textbox_collapse_selection (GeedotWidget *widget) {
	int char_width = XTextWidth(widget->font, " ", 1);
	int start_offset = 2;
	int strloc = 0;
	int start_selection = (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start - start_offset) / char_width;
	int stop_selection = (GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop - start_offset) / char_width;
	char *text;
	
	if (start_selection > stop_selection) {
		strloc = stop_selection;
		stop_selection = start_selection;
		start_selection = strloc;
		strloc = 0;
	}
	
	if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start != GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop) {
		text = (char *) malloc((start_selection + (strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) - stop_selection) + 1) * sizeof(char));
		memcpy(text, GEEDOT_TEXTBOX(widget->widget_data)->text, start_selection * sizeof(char));
		for (strloc = 0; strloc < strlen(GEEDOT_TEXTBOX(widget->widget_data)->text) - stop_selection; strloc++) {
			text[strloc + start_selection] = GEEDOT_TEXTBOX(widget->widget_data)->text[strloc + stop_selection];
		}
		text[strloc + start_selection] = '\0';
		gdot_textbox_set_text(widget, text);
		
      if (GEEDOT_TEXTBOX(widget->widget_data)->cursor_start > GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop) {
	      GEEDOT_TEXTBOX(widget->widget_data)->cursor_start = GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop;
      }
		else {
			GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop = GEEDOT_TEXTBOX(widget->widget_data)->cursor_start;
		}
		while (GEEDOT_TEXTBOX(widget->widget_data)->view_start > GEEDOT_TEXTBOX(widget->widget_data)->cursor_stop + char_width + 2) {
			GEEDOT_TEXTBOX(widget->widget_data)->view_start--;
		}
		if (GEEDOT_TEXTBOX(widget->widget_data)->view_start < char_width + 2) {
			GEEDOT_TEXTBOX(widget->widget_data)->view_start = 0;
		}
		gdot_redraw_widget(widget);
		
		free(text);
		return 1;
	}
	else {
		return 0;
	}
}
