/* Geedot Toolkit - Widget set for Geedot Web Browser
 *   File: gdtoolkitwidget.c - generic widget handling functions
 *   Author: Luke Crowthers
 *   Start Date: Aug, 2000
 *   Version: 0.1
 *   License: GPL
 */

#include <gdtoolkit.h>
#include <sys/timeb.h>

GeedotWidget *__focused_widget = NULL;
GeedotWidget *__depression_focused_widget = NULL;
GeedotWidget *__motion_focused_widget = NULL;
GeedotWidget *__keyboard_focused_widget = NULL;

GeedotWidget *gdot_focused_widget(void) {
	return __focused_widget;
}

GeedotWidget *gdot_depression_focused_widget(void) {
	return __depression_focused_widget;
}

GeedotWidget *gdot_motion_focused_widget(void) {
	return __motion_focused_widget;
}

GeedotWidget *gdot_keyboard_focused_widget(void) {
	return __keyboard_focused_widget;
}

int gdot_widget_verify_motion_focus (int x, int y) {
	GeedotWidget *widget_holder;

	if (__motion_focused_widget != NULL) {
		if (!gdot_area_bounds_location(gdot_widget_absolute_area(__motion_focused_widget), x, y)) {
			widget_holder = __motion_focused_widget;
			__motion_focused_widget = NULL;
			__depression_focused_widget = NULL;
			gdot_redraw_widget(widget_holder);
			return 0;
		}
		else {
			return 0;
		}
	}
	return 0;
}

int gdot_widget_verify_focus (int x, int y) {
	GeedotWidget *widget_holder;
	if (__focused_widget != NULL) {
		if (!gdot_area_bounds_location(gdot_widget_absolute_area(__focused_widget), x, y)) {
			widget_holder = __focused_widget;
			__focused_widget = NULL;
			gdot_redraw_widget(widget_holder);
			return 0;
		}
		else {
			return 0;
		}
	}
}

int gdot_widget_clear_focus (void) {
	GeedotWidget *widget_holder;

	if (__motion_focused_widget != NULL) {
		widget_holder = __motion_focused_widget;
		__motion_focused_widget = NULL;
		__depression_focused_widget = NULL;
		gdot_redraw_widget(widget_holder);
	}
}

int gdot_widget_grab_focus (GeedotWidget *focus) {
	if (__focused_widget != focus) {
		if (__focused_widget == NULL) {
			__focused_widget = focus;
		}
	}
	if (__motion_focused_widget != focus) {
		__motion_focused_widget = focus;
	}
	if (__keyboard_focused_widget != focus) {
		__keyboard_focused_widget = focus;
	}
	return 1;
}

int gdot_widget_steal_focus (GeedotWidget *focus) {
	if (__focused_widget != focus) {
		__focused_widget = focus;
	}
	if (__keyboard_focused_widget != focus) {
		__keyboard_focused_widget = focus;
	}
	gdot_redraw_widget(focus);
	return 1;
}

int gdot_widget_steal_motion_focus (GeedotWidget *focus) {
	GeedotWidget *old;
	puts("stealing motion focus");
	old = __motion_focused_widget = NULL;
	gdot_redraw_widget(old);
	gdot_widget_grab_motion_focus(focus);
}

int gdot_widget_grab_motion_focus (GeedotWidget *focus) {
	GeedotWidget *holder;
	if (__motion_focused_widget != focus) {
		if (__motion_focused_widget != NULL) {
			if (gdot_widget_is_parent_of_widget(focus, __motion_focused_widget)) {
				return 0;
			}
			holder = __motion_focused_widget;
			__motion_focused_widget = NULL;
			gdot_redraw_widget(holder);
		}
		__motion_focused_widget = focus;
		gdot_redraw_widget(focus);
	}
	return 1;
}

int gdot_widget_grab_depression_focus (GeedotWidget *focus) {
	if (__depression_focused_widget != focus) {
		__depression_focused_widget = focus;
	}
	return 1;
}

int gdot_widget_has_focus (GeedotWidget *widget) {
	if (__focused_widget == widget) {
		return 1;
	}
	return 0;
}

int gdot_widget_has_motion_focus (GeedotWidget *widget) {
	if (__motion_focused_widget == widget) {
		return 1;
	}
	return 0;
}

int gdot_widget_has_depression_focus (GeedotWidget *widget) {
	if (__depression_focused_widget == widget) {
		return 1;
	}
	return 0;
}

int gdot_widget_has_keyboard_focus (GeedotWidget *widget) {
	if (__keyboard_focused_widget == widget) {
		return 1;
	}
	return 0;
}

int gdot_widget_release_focus (GeedotWidget *widget) {
	if (__focused_widget != NULL) {
		if (__focused_widget == widget) {
			__focused_widget = NULL;
			return 1;
		}
		else {
			return -1;
		}
	}

	return 0;
}

int gdot_redraw_widget (GeedotWidget *widget) {
	if (widget->is_visible) {
		widget->draw_package.redraw(widget->draw_package.redraw_data);
		gdot_redraw_widget_recursive(widget);
	}
}

int gdot_redraw_widget_recursive (GeedotWidget *widget) {
	GeedotWidgetList *widget_holder;

	widget_holder = widget->first_widget;
	while (widget_holder != widget->last_widget) {
		gdot_redraw_widget(widget_holder->widget);
		widget_holder = widget_holder->next;
	}
}

int gdot_shade_widget (GeedotWidget *widget) {
	int x, y;

	XSetForeground(widget->parent->display, widget->parent->frame_gc, WhitePixel(widget->parent->display, DefaultScreen(widget->parent->display)));
	for (x = gdot_widget_absolute_x(widget); x <= gdot_widget_absolute_x(widget) + widget->area.width; x++) {
		if ( ((float) x / 2) == ((int) x / 2) ) {
			for (y = gdot_widget_absolute_y(widget) + 1; y <= gdot_widget_absolute_y(widget) + widget->area.height; y += 2) {
				XDrawPoint(widget->parent->display, widget->parent->backbuffer, widget->parent->frame_gc, x, y);
			}
		}
		else {
			for (y = gdot_widget_absolute_y(widget); y <= gdot_widget_absolute_y(widget) + widget->area.height; y += 2) {
				XDrawPoint(widget->parent->display, widget->parent->backbuffer, widget->parent->frame_gc, x, y);
			}
		}
	}
	XCopyArea(widget->parent->display, widget->parent->backbuffer, widget->parent->drawable, widget->parent->frame_gc, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget), widget->area.width + 1, widget->area.height + 1, gdot_widget_absolute_x(widget), gdot_widget_absolute_y(widget));
}

int gdot_widget_set_active (GeedotWidget *widget, Bool state) {
	widget->is_active = state;
	gdot_redraw_widget(widget);
}

int gdot_widget_set_visible (GeedotWidget *widget, Bool state) {
	widget->is_visible = state;
	gdot_redraw_widget(widget);
}

int gdot_widget_run_actions_for_event (GeedotWidget *widget, XEvent event) {
	GeedotActionList *action_list = widget->first_widget_action;

	if (widget->drawable != event.xany.window) {
		return 0;
	}
	switch (event.type) {
	 case ButtonPress:
		if (widget->is_active && gdot_area_bounds_location(gdot_widget_absolute_area(widget), event.xbutton.x, event.xbutton.y)) {
			gdot_widget_steal_focus(widget);
			gdot_widget_grab_depression_focus(widget);
			gdot_redraw_widget(widget);
			if (gdot_widget_has_focus(widget)) {
				while (action_list != widget->last_widget_action) {
					if (action_list->action->trigger_mask & GeedotButtonPress) {
						action_list->action->response(action_list->action->response_data, &event);
					}
					action_list = action_list->next;
				}
			}
		}
		break;
	 case ButtonRelease:
		gdot_widget_verify_focus(event.xbutton.x, event.xbutton.y - gdot_widget_absolute_y(widget->parent));
		gdot_reset_pointer(widget);
		if (widget->is_active) {
			if (gdot_area_bounds_location(gdot_widget_absolute_area(widget), event.xbutton.x, event.xbutton.y)) {
				gdot_widget_grab_motion_focus(widget);
				gdot_redraw_widget(widget);
				while (action_list != widget->last_widget_action) {
					if (action_list->action->trigger_mask & GeedotButtonRelease) {
						action_list->action->response(action_list->action->response_data, &event);
					}
					if (action_list->action->trigger_mask & GeedotButtonPressRelease && gdot_widget_has_depression_focus) {
						action_list->action->response(action_list->action->response_data, &event);
					}
					action_list = action_list->next;
				}
				gdot_widget_release_focus(widget);
			}
		}
		break;
	 case MotionNotify:
		if (widget->is_active) {
			if (gdot_area_bounds_location(gdot_widget_absolute_area(widget), event.xbutton.x, event.xbutton.y) && widget->type != GEEDOT_TOPLEVEL_FRAME && widget->type != GEEDOT_CHILD_FRAME && widget->type != GEEDOT_POPUP_FRAME){
				gdot_widget_grab_motion_focus(widget);
			}
			if (gdot_widget_has_focus(widget)) {
				while (action_list != widget->last_widget_action) {
					if (action_list->action->trigger_mask & GeedotPointerMotion) {
						action_list->action->response(action_list->action->response_data, &event);
					}
					action_list = action_list->next;
				}
			}
		}
		break;
	 case KeyPress:
		if (gdot_widget_has_keyboard_focus(widget) && widget->is_active) {
			action_list = widget->first_widget_action;
			while (action_list != widget->last_widget_action) {
				if (action_list->action->trigger_mask & GeedotKeyPress) {
					action_list->action->response(action_list->action->response_data, &event);
				}
				action_list = action_list->next;
			}
		}
		break;
	 case KeyRelease:
		if (gdot_widget_has_keyboard_focus(widget) && widget->is_active) {
			action_list = widget->first_widget_action;
			while (action_list != widget->last_widget_action) {
				if (action_list->action->trigger_mask & GeedotKeyRelease) {
					action_list->action->response(action_list->action->response_data, &event);
				}
				action_list = action_list->next;
			}
		}
		break;
	 case Expose:
		if (widget->is_active) {
			while (action_list != widget->last_widget_action) {
				if (action_list->action->trigger_mask & GeedotExpose) {
					action_list->action->response(action_list->action->response_data, &event);
				}
				action_list = action_list->next;
			}
		}
		break;
	 case EnterNotify:
		if (widget->is_active) {
			while (action_list != widget->last_widget_action) {
				if (action_list->action->trigger_mask & GeedotFrameEnter) {
					action_list->action->response(action_list->action->response_data, &event);
				}
				action_list = action_list->next;
			}
		}
		break;
	 case LeaveNotify:
		if (widget->is_active) {
			while (action_list != widget->last_widget_action) {
				if (action_list->action->trigger_mask & GeedotFrameLeave) {
					action_list->action->response(action_list->action->response_data, &event);
				}
				action_list = action_list->next;
			}
		}
		break;
	 case ConfigureNotify:
		if (widget->is_active) {
			while (action_list != widget->last_widget_action) {
				if (action_list->action->trigger_mask & GeedotFrameConfigure) {
					action_list->action->response(action_list->action->response_data, &event);
				}
				action_list = action_list->next;
			}
		}
		break;
	}
	
	switch (event.type) {
	 case ButtonPress:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (gdot_area_bounds_location(action_list->action->area, event.xbutton.x - gdot_widget_absolute_x(widget->parent), event.xbutton.y - gdot_widget_absolute_y(widget->parent)) && (action_list->action->trigger_mask & GeedotButtonPress)) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case ButtonRelease:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (gdot_area_bounds_location(action_list->action->area, event.xbutton.x - gdot_widget_absolute_x(widget->parent), event.xbutton.y - gdot_widget_absolute_y(widget->parent)) && (action_list->action->trigger_mask & GeedotButtonRelease)) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case MotionNotify:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (gdot_area_bounds_location(action_list->action->area, event.xbutton.x - gdot_widget_absolute_x(widget->parent), event.xbutton.y - gdot_widget_absolute_y(widget->parent)) && (action_list->action->trigger_mask & GeedotPointerMotion)) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case Expose:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (action_list->action->trigger_mask & GeedotExpose) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case EnterNotify:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (action_list->action->trigger_mask & GeedotFrameEnter) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case LeaveNotify:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (action_list->action->trigger_mask & GeedotFrameLeave) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case ConfigureNotify:
		action_list = widget->first_action;
		while (action_list != widget->last_action) {
			if (action_list->action->trigger_mask & GeedotFrameConfigure) {
				action_list->action->response(action_list->action->response_data, &event);
			}
			action_list = action_list->next;
		}
		break;
	 case ClientMessage:
		return 0;
	}
}

int gdot_widget_run_actions_recursive (GeedotWidget *widget, XEvent event) {
	GeedotWidgetList *widget_list = widget->first_widget;

	while (widget_list != widget->last_widget) {
		gdot_widget_run_actions_for_event (widget_list->widget, event);
		gdot_widget_run_actions_recursive (widget_list->widget, event);
		widget_list = widget_list->next;
	}
	return 1;
}

int gdot_widget_destroy (GeedotWidget *widget) {
	GeedotWidgetList *widget_list;

	if (widget->parent != NULL) {
		widget_list = widget->parent->first_widget;
		if (widget == widget_list->widget) {
			widget->parent->first_widget = widget->parent->first_widget->next;
		}
		else {
			while (widget != widget_list->next->widget) {
				widget_list = widget_list->next;
			}
			widget_list->next = widget_list->next->next;
		}
	}
	gdot_widget_destroy_recursive(widget);
}

int gdot_widget_destroy_recursive (GeedotWidget *widget) {
	GeedotWidgetList *widget_list;
	GeedotWidgetList *widget_list_holder;

	widget_list = widget->first_widget;
	while (widget_list != widget->last_widget) {
		gdot_widget_destroy(widget_list->widget);
		widget_list_holder = widget_list->next;
		if (widget->parent == NULL) {
			XFreeColormap(widget->display, widget->colormap);
			free(widget->geedot_colors);
			XFreePixmap(widget->display, widget->backbuffer);
			XDestroyWindow(widget->display, widget->drawable);
		}
		else {
			if (widget->font != widget->parent->font) {
				XFreeFont(widget->display, widget->font);
			}
			if (widget->colormap != widget->parent->colormap) {
				XFreeColormap(widget->display, widget->colormap);
			}
			if (widget->geedot_colors != widget->parent->geedot_colors) {
				free(widget->geedot_colors);
			}
		}
		free(widget_list->widget);
		free(widget_list);
		widget_list = widget_list_holder;
	}
	free(widget->last_widget->widget);
	free(widget->last_widget);
	free(widget);
}

int gdot_widget_inherit (GeedotWidget *parent, GeedotWidget *child) {
	child->display = parent->display;
	child->drawable = parent->drawable;
	child->backbuffer = parent->backbuffer;
	child->frame_gc = parent->frame_gc;
	child->font = parent->font;
	child->selection = parent->selection;
	child->pointer_state = parent->pointer_state;
	child->colormap = parent->colormap;
	child->geedot_colors = (GeedotDefaultColorSet *) malloc(sizeof(GeedotDefaultColorSet));
	XAllocNamedColor(child->display, child->colormap, "rgb:a8/a5/cc", &child->geedot_colors->light1.color, &child->geedot_colors->light1.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:57/52/7c", &child->geedot_colors->mid1.color, &child->geedot_colors->mid1.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:09/00/4f", &child->geedot_colors->dark1.color, &child->geedot_colors->dark1.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:ea/ea/ea", &child->geedot_colors->light2.color, &child->geedot_colors->light2.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:aa/aa/aa", &child->geedot_colors->mid2.color, &child->geedot_colors->mid2.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:8c/8c/8c", &child->geedot_colors->dark2.color, &child->geedot_colors->dark2.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:00/00/00", &child->geedot_colors->foreground.color, &child->geedot_colors->foreground.color);
	XAllocNamedColor(child->display, child->colormap, "rgb:ff/ff/ff", &child->geedot_colors->background.color, &child->geedot_colors->background.color);

	child->parent = parent;
	return 1;
}

int gdot_widget_reinherit_recursive (GeedotWidget *widget) {
	GeedotWidgetList *widget_list = widget->first_widget;
	while (widget_list != widget->last_widget) {
		if (widget_list->widget->type != GEEDOT_TOPLEVEL_FRAME && widget_list->widget->type != GEEDOT_POPUP_FRAME) {
			gdot_widget_inherit(widget, widget_list->widget);
			widget_list->widget->drawable = widget->drawable;
			widget_list->widget->backbuffer = widget->backbuffer;
			gdot_widget_reinherit_recursive(widget_list->widget);
		}
		widget_list = widget_list->next;
	}
}

int gdot_widget_absolute_x (GeedotWidget *widget) {
	int absolute_x = 0;

	if (widget != NULL) {
		while (widget->parent != NULL) {
			absolute_x += widget->area.x;
			widget = widget->parent;
		}
	}
	return absolute_x;
}

int gdot_widget_absolute_y (GeedotWidget *widget) {
	int absolute_y = 0;

	if (widget != NULL) {
		while (widget->parent != NULL) {
			absolute_y += widget->area.y;
			widget = widget->parent;
		}
	}
	return absolute_y;
}

GeedotArea gdot_widget_absolute_area (GeedotWidget *widget) {
	GeedotArea absolute_area;

	absolute_area.x = gdot_widget_absolute_x(widget);
	absolute_area.y = gdot_widget_absolute_y(widget);
	absolute_area.width = widget->area.width;
	absolute_area.height = widget->area.height;

	return absolute_area;
}

int gdot_widget_is_parent_of_widget (GeedotWidget *parent, GeedotWidget *child) {
	while (child->parent != NULL) {
		if (child->parent == parent) {
			return 1;
		}
		else {
			child = child->parent;
		}
	}
	return 0;
}

void gdot_null_function (void) {
}

GeedotArea gdot_widget_fit (GeedotWidget *widget) {
	GeedotArea child_max_bounds;
	GeedotWidgetList *widget_holder;

	widget_holder = widget->first_widget;
	while (widget_holder != widget->last_widget && (widget_holder->widget->type != GEEDOT_TOPLEVEL_FRAME && widget_holder->widget->type != GEEDOT_POPUP_FRAME)) {
		gdot_widget_fit(widget_holder->widget);
		if (widget_holder->widget->area.x + widget_holder->widget->area.width >= widget->area.width) {
			widget->area.width = widget_holder->widget->area.x + widget_holder->widget->area.width + 1;
		}
		if (widget_holder->widget->area.y + widget_holder->widget->area.height >= widget->area.height) {
			widget->area.height = widget_holder->widget->area.y + widget_holder->widget->area.height + 1;
		}
		widget_holder = widget_holder->next;
	}
}

GeedotArea gdot_widget_shrinkwrap (GeedotWidget *widget) {
	widget->area.width = 0;
	widget->area.height = 0;
	gdot_widget_fit(widget);
}

GeedotWidget *gdot_toplevel_widget (GeedotWidget *widget) {
	GeedotWidget *toplevel = widget;

	while (toplevel->parent != NULL) {
		toplevel = toplevel->parent;
	}
	return toplevel;
}
